#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Written (W) 2006-2008 Soeren Sonnenburg
# Written (W) 2006-2007 Mikio Braun
# Copyright (C) 2007 Fraunhofer Institute FIRST and Max-Planck-Society
#

import time
from string import maketrans

""" this function is 100% compatible to the matlab function, thus it is one based (!)
    use one_based=False if needed, then however the interval is [start,stop) (excluding stop)
"""
def load_genomic(chromosome, strand, start, stop, genome, one_based=True):
    fname = '/fml/ag-raetsch/share/databases/genomes/' + genome + '/' + chromosome[3:] + '.flat'
    f = file(fname)
    if one_based:
        f.seek(start - 1)
        str = f.read(stop - start + 1)
    else:
        f.seek(start)
        str = f.read(stop - start)

    if strand == '-':
        return reverse_complement(str)
    elif strand == '+':
        return str
    else:
        print 'strand must be + or -'
        raise KeyError

""" read a table browser ascii output file (http://genome.ucsc.edu/cgi-bin/hgTables) """
def read_table_browser(f):
    table = dict();
    for l in f.readlines():
        if not l.startswith('#'):
            (name, chrom, strand, txStart, txEnd, cdsStart, cdsEnd, exonCount, exonStarts, exonEnds, proteinID, alignID) = l.split('\t')
            exonStarts = [ int(i) for i in exonStarts.split(',')[:-1] ]
            exonEnds = [ int(i) for i in exonEnds.split(',')[:-1] ]

            table[name] = { 'chrom': chrom, 'strand': strand, 'txStart': int(txStart), 'txEnd': int(txEnd),
            'cdsStart': int(cdsStart), 'cdsEnd': int(cdsEnd), 'exonCount': int(exonCount), 'exonStarts': exonStarts,
            'exonEnds': exonEnds, 'proteinID': proteinID, 'alignID': alignID[:-1] }

    return table

""" get promoter region """
def get_promoter_region(chromosome, strand, gene_start, gene_end, genome, length):

    if strand == '+':
        return load_genomic(chromosome, strand, gene_start, gene_start + length, genome, one_based=False)
    elif strand == '-':
        return load_genomic(chromosome, strand, gene_end, gene_end + length, genome, one_based=False)
    else:
        print 'unknown strand'
        return None

""" reverse + complement a DNA sequence (only letters ACGT are translated!)
    FIXME won't work with all the rest like y... """
def reverse_complement(str):
    t = maketrans('acgtACGT', 'tgcaTGCA')
    return str[len(str)::-1].translate(t)

""" works only with .fa files that contain a single entry """
def read_single_fasta(fname):
    str = file(fname).read()
    str = str[str.index('\n') + 1:].replace('\n', '')
    return str

""" writes only single enty .fa files """
def write_single_fasta(fname, name, str, linelen=60):
    header = '>' + name + '\n'
    f = file(fname, 'a')
    f.write(header)
    for i in xrange(0, len(str), linelen):
        f.write(str[i:i + linelen] + '\n')
    f.close()

""" read fasta as dictionary """
def read_fasta(f):
    fasta = dict()

    for s in f.readlines():
        if s.startswith('>'):
            key = s[1:-1]
            fasta[key] = ""
        else:
            fasta[key] += s[:-1]

    return fasta

""" write dictionary fasta """
def write_fasta(f, d, linelen=60):
    for k in sorted(d):
        f.write('>%s\n' % k);
        s = d[k]
        for i in xrange(0, len(s), linelen):
            f.write(s[i:i + linelen] + '\n')

def write_gff(f, (source, version), (seqtype, seqname), descrlist, skipheader=False):
    """ writes a gff version 2 file
        descrlist is a list of dictionaries, each of which contain these fields:
        <seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
    """

    if not skipheader:
        f.write('##gff-version 2\n')
        f.write('##source-version %s %s\n' % (source, version))

        t = time.localtime()
        f.write("##date %d-%d-%d %d:%d:%d\n" % t[0:6])

    f.write('##Type %s %s\n' % (seqtype, seqname))

    for d in descrlist:
        f.write('%s\t%s\t%s\t%d\t%d\t%+f\t%s\t%d' % (d['seqname'], d['source'],
                                            d['feature'], d['start'], d['end'],
                                            d['score'], d['strand'], d['frame']))
        if d.has_key('attributes'):
            f.write('\t' + d['attributes'])
            if d.has_key('comments'):
                f.write('\t' + d['comments'])
        f.write('\n')


if __name__ == '__main__':
    import sys, os

    table = read_table_browser(file('/fml/ag-raetsch/home/sonne/addnet/tfbs/share/data/wt1_bibliosphere_table_browser_hg17.txt'))
    print table.keys()
    print table[table.keys()[0]]
    d = { 'ahoernchen' : 'ACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGTACGT',
          'bhoernchen' : 'GATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACAGATTACA' }

    write_fasta(sys.stdout, d)
    write_fasta(file('/tmp/test.fa', 'w'), d)

    d2 = read_fasta(file('/tmp/test.fa'))
    os.unlink('/tmp/test.fa')

    print d
    print d2
    print d == d2

    p = load_genomic('chr5', '+', 100000, 100100, 'hg17')
    n = load_genomic('chr1', '-', 3000000, 3001000, 'mm7')
    write_single_fasta('bla.fa', 'bla', 'ACGT')
    n2 = read_single_fasta('bla.fa')



#!/usr/bin/env python

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2008-2009 Soeren Sonnenburg
# Copyright (C) 2008-2009 Fraunhofer Institute FIRST and Max Planck Society

class_str='class'
types=["BOOL", "CHAR", "INT8", "UINT8", "INT16", "UINT16", "INT32", "UINT32",
		"INT64", "UINT64", "FLOAT32", "FLOAT64", "FLOATMAX", "COMPLEX128"]
config_tests=["HAVE_HDF5", "HAVE_JSON", "HAVE_XML", "HAVE_LAPACK", "USE_CPLEX",
	"USE_SVMLIGHT", "USE_GLPK", "USE_LZO", "USE_GZIP", "USE_BZIP2", "USE_LZMA",
	"USE_MOSEK", "HAVE_EIGEN3", "HAVE_COLPACK", "HAVE_NLOPT", "HAVE_PROTOBUF"]

SHOGUN_TEMPLATE_CLASS = "SHOGUN_TEMPLATE_CLASS"
SHOGUN_BASIC_CLASS = "SHOGUN_BASIC_CLASS"

def check_class(line):
	if not (line.find('public')==-1 and
			line.find('private')==-1 and
			line.find('protected')==-1):
		return True

def check_abstract_class(line):
	line=line.replace(' ','').replace('\t','').strip()
	return line.endswith('=0;')

def check_is_in_blacklist(c, lines, line_nr, blacklist):
	ifdef_cnt=0
	for i in range(line_nr,0,-1):
		line=lines[i]
		if line.find('#endif')!=-1:
			ifdef_cnt-=1
		if line.find('#ifdef')!=-1:
			ifdef_cnt+=1

			for b in blacklist.keys():
				if line.find(b)!=-1 and ifdef_cnt>0:
					return True
		if line.find('#ifndef')!=-1:
			ifdef_cnt+=1

	return False

def extract_class_name(lines, line_nr, line, blacklist):
	try:
		if not line:
			line=lines[line_nr]
		c=line[line.index(class_str)+len(class_str):]
		if not ':' in c:
			return
		if not check_class(line):
			if not check_class(lines[line_nr+1]):
				return
		c=c.split()[0]
	except:
		return

	c=c.strip(':').strip()

	if not c.startswith('C'):
		return
	if c.endswith(';'):
		return
	if '>' in c:
		return
	if not (len(c)>2 and c[1].isupper()):
		return
	if check_is_in_blacklist(c[1:], lines, line_nr, blacklist):
		return

	return c[1:]

def get_includes(classes):
	class_headers = []
	for c,t in classes:
		class_headers.append(c+".h")

	import os
	result = []
	for root, dirs, files in os.walk("."):
		for f in files:
			if f in class_headers:
				result.append(os.path.join(root, f))

	includes=[]
	for o in result:
		includes.append('#include "%s"' % o.strip().lstrip('./'))
	return includes

def get_definitions(classes):
	definitions=[]
	definitions.append("#define %s" % SHOGUN_TEMPLATE_CLASS)
	definitions.append("#define %s" % SHOGUN_BASIC_CLASS)
	for c,t in classes:
		d="static %s CSGObject* __new_C%s(EPrimitiveType g) { return g == PT_NOT_GENERIC? new C%s(): NULL; }" % (SHOGUN_BASIC_CLASS,c,c)
		definitions.append(d)
	return definitions

def get_template_definitions(classes, supports_complex):
	definitions=[]
	for c,t in classes:
		d=[]
		d.append("static %s CSGObject* __new_C%s(EPrimitiveType g)\n{\n\tswitch (g)\n\t{\n" % (SHOGUN_TEMPLATE_CLASS,c))
		for t in types:
			if t in ('BOOL','CHAR'):
				suffix=''
			else:
				suffix='_t'
			if t=='COMPLEX128' and not supports_complex:
				d.append("\t\tcase PT_COMPLEX128: return NULL;\n")
			else:
				d.append("\t\tcase PT_%s: return new C%s<%s%s>();\n" % (t,c,t.lower(),suffix))
		d.append("\t\tcase PT_SGOBJECT:\n")
		d.append("\t\tcase PT_UNDEFINED: return NULL;\n\t}\n\treturn NULL;\n}")
		definitions.append(''.join(d))
	return definitions

def get_struct(classes):
	struct=[]
	for c,template in classes:
		prefix = SHOGUN_BASIC_CLASS
		if template:
			prefix = SHOGUN_TEMPLATE_CLASS

		s='{"%s", %s __new_C%s},' % (c,prefix,c)
		struct.append(s)
	return struct

def extract_block(c, lines, start_line, stop_line, start_sym, stop_sym):
	sym_cnt=0

	block_start=-1;
	block_stop=-1;

	for line_nr in range(start_line, stop_line):
		line=lines[line_nr]
		if line.find(start_sym)!=-1:
			sym_cnt+=1
			if block_start==-1:
				block_start=line_nr
		if line.find(stop_sym)!=-1:
			block_stop=line_nr+1
			sym_cnt-=1
		if sym_cnt==0 and block_start!=-1 and block_stop!=-1:
			return block_start,block_stop

	return block_start,block_stop

def check_complex_supported_class(line):
	l=list(filter(lambda y:y if y!='' else None,\
		line.strip().replace('\t',' ').split(' ')))
	supported=len(l)==3 and l[0]=='typedef' and l[1]=='bool'\
		and l[2]=='supports_complex128_t;'
	return supported

def test_candidate(c, lines, line_nr, supports_complex):
	start,stop=extract_block(c, lines, line_nr, len(lines), '{','}')
	if stop<line_nr:
		return False, line_nr+1
	complex_supported=False
	for line_nr in range(start, stop):
		line=lines[line_nr]
		if line.find('virtual')!=-1:
			if check_abstract_class(line):
				return False, stop
			else:
				vstart,vstop=extract_block(c, lines, line_nr, stop, '(',')')
				for line_nr in range(vstart, vstop):
					line=lines[line_nr]
					if check_abstract_class(line):
						return False, stop
		if line.find('supports_complex128_t')!=-1:
			if check_complex_supported_class(line):
				complex_supported=True
				if not supports_complex:
					return False, stop
	if supports_complex and not complex_supported:
		return False, stop

	return True, stop


def extract_classes(HEADERS, template, blacklist, supports_complex):
	"""
	Search in headers for non-template/non-abstract class-names starting
	with `C'.

	Does not support local nor multiple classes and
	drops classes with pure virtual functions
	"""
	classes=list()
	for fname in HEADERS:
		lines=open(fname).readlines()
		line_nr=0
		while line_nr<len(lines):
			line=lines[line_nr]

			if line.find('IGNORE_IN_CLASSLIST')!=-1:
				line_nr+=1
				continue
			c=None
			if template:
				tp=line.find('template')
				if tp!=-1:
					line=line[tp:]
					cp=line.find('>')
					line=line[cp+1:]
					cp=line.find(class_str)
					if cp!=-1:
						c=extract_class_name(lines, line_nr, line, blacklist)
			else:
				if line.find(class_str)!=-1:
					c=extract_class_name(lines, line_nr, None, blacklist)
			if c:
				ok, line_nr=test_candidate(c, lines, line_nr, supports_complex)
				if ok:
					classes.append((c,template))
				continue

			line_nr+=1
	return classes


def write_templated_file(fname, substitutes):
	template=open(fname).readlines()

	f=open(fname,'w')
	for line in template:
		l=line.strip()
		if l.startswith('REPLACE') and l.endswith('THIS'):
			l=line.split()[1]
			if sys.version_info >= (3,):
				for s in substitutes.keys():
					if l==s:
						f.write('\n'.join(substitutes[s]))
					continue
			else:
				for s in substitutes.iterkeys():
					if l==s:
						f.write('\n'.join(substitutes[s]))
					continue
		else:
			f.write(line)


def read_config():
	config=dict()
	for line in open('lib/config.h').readlines():
		if line=='\n':
			continue
		l=[l.strip() for l in line.split()]
		config[l[1]]=1

	return config

def get_blacklist():
	config=read_config()
	blacklist=dict()
	for cfg in config_tests:
		if not cfg in config:
			blacklist[cfg]=1
	return blacklist

if __name__=='__main__':
	import sys
	TEMPL_FILE=sys.argv[1]
	HEADERS=None
	if (sys.argv[2] == "-in"):
		# read header file list from file
		with open(sys.argv[3]) as f:
			content = f.readlines()
			HEADERS = [x.strip() for x in content]
	else:
		HEADERS=sys.argv[2:]

	blacklist = get_blacklist()

	classes = extract_classes(HEADERS, False, blacklist, False)
	template_classes = extract_classes(HEADERS, True, blacklist, False)
	complex_template_classes = extract_classes(HEADERS, True, blacklist, True)
	includes = get_includes(classes+template_classes+complex_template_classes)
	definitions = get_definitions(classes)
	template_definitions = get_template_definitions(template_classes, False)
	complex_template_definitions = get_template_definitions(complex_template_classes, True)
	struct = get_struct(classes+template_classes+complex_template_classes)
	substitutes = {'includes': includes,
		'definitions' :definitions,
		'template_definitions' : template_definitions,
		'complex_template_definitions' : complex_template_definitions,
		'struct' : struct
		}

	write_templated_file(TEMPL_FILE, substitutes)



#!/usr/bin/env python
parameter_list=[['../data/snps.dat']]

def features_snp_modular (fname):
	from modshogun import StringByteFeatures, SNPFeatures, SNP

	sf=StringByteFeatures(SNP)
	sf.load_ascii_file(fname, False, SNP, SNP)
	#print(sf.get_features())
	snps=SNPFeatures(sf)
	#print(snps.get_feature_matrix())
	#print(snps.get_minor_base_string())
	#print(snps.get_major_base_string())

if __name__=='__main__':
	print('SNP Features')
	features_snp_modular(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
parameter_list=[[traindna,testdna,10,3,0,'n',False,'FULL'],
		[traindna,testdna,11,4,0,'n',False,'FULL']]

def preproc_sortulongstring (fm_train_dna=traindna,fm_test_dna=testdna,
				 size_cache=10,order=3,gap=0,reverse='n',
				 use_sign=False,normalization='FULL'):

	sg('add_preproc', 'SORTULONGSTRING')
	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('convert', 'TRAIN', 'STRING', 'CHAR', 'STRING', 'ULONG', order, order-1, gap, reverse)
	sg('attach_preproc', 'TRAIN')

	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('convert', 'TEST', 'STRING', 'CHAR', 'STRING', 'ULONG', order, order-1, gap, reverse)
	sg('attach_preproc', 'TEST')

	sg('set_kernel', 'COMMSTRING', 'ULONG', size_cache, use_sign, normalization)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('CommUlongString')
	preproc_sortulongstring(*parameter_list[0])



#!/usr/bin/env python
from numpy import *
parameter_list = [[10,3,15,2.1,1,1e-5,1],[20,4,15,2.2,2,1e-5,2]]

def classifier_multiclassocas_modular (num_vec=10,num_class=3,distance=15,width=2.1,C=1,epsilon=1e-5,seed=1):
	from modshogun import RealFeatures, MulticlassLabels
	from modshogun import MulticlassOCAS
	from modshogun import Math_init_random

	# reproducible results
	random.seed(seed)
	Math_init_random(seed)

	# generate some training data where each class pair is linearly separable
	label_train=array([mod(x,num_class) for x in range(num_vec)],dtype="float64")
	label_test=array([mod(x,num_class) for x in range(num_vec)],dtype="float64")
	fm_train=array(random.randn(num_class,num_vec))
	fm_test=array(random.randn(num_class,num_vec))
	for i in range(len(label_train)):
		fm_train[label_train[i],i]+=distance
		fm_test[label_test[i],i]+=distance

	feats_train=RealFeatures(fm_train)
	feats_test=RealFeatures(fm_test)

	labels=MulticlassLabels(label_train)

	classifier = MulticlassOCAS(C,feats_train,labels)
	classifier.train()

	out = classifier.apply(feats_test).get_labels()
	#print label_test
	#print out
	return out,classifier

if __name__=='__main__':
	print('MulticlassOCAS')
	classifier_multiclassocas_modular(*parameter_list[0])



"""
Common operations on features
"""

import shogun.Features as features
import shogun.Preprocessor as preproc

WORDSTRING_ORDER=3
WORDSTRING_GAP=0
WORDSTRING_REVERSE=False


def get_features(fclass, ftype, data, *args, **kwargs):
	if fclass=='simple':
		return get_simple(ftype, data, *args, **kwargs)
	elif fclass=='string':
		return get_string(ftype, data, *args, **kwargs)
	elif fclass=='string_complex':
		return get_string_complex(ftype, data, *args, **kwargs)
	elif fclass=='wd':
		return get_wd(data, *args, **kwargs)
	else:
		raise ValueError, 'Unknown feature class %s.'%fclass


def get_simple (ftype, data, alphabet=features.DNA, sparse=False):
	"""Return SimpleFeatures.

	@param ftype Feature type, e.g. Real, Byte
	@param data Train/test data for feature creation
	@param alphabet Alphabet for feature creation
	@param sparse Is feature sparse?
	@return Dict with SimpleFeatures train/test
	"""

	if ftype=='Byte' or ftype=='Char':
		train=eval('features.'+ftype+'Features(alphabet)')
		test=eval('features.'+ftype+'Features(alphabet)')
		train.copy_feature_matrix(data['train'])
		test.copy_feature_matrix(data['test'])

	else:
		train=eval('features.'+ftype+"Features(data['train'])")
		test=eval('features.'+ftype+"Features(data['test'])")

	if sparse:
		sparse_train=eval('features.Sparse'+ftype+'Features()')
		sparse_train.obtain_from_simple(train)

		sparse_test=eval('features.Sparse'+ftype+'Features()')
		sparse_test.obtain_from_simple(test)

		return {'train':sparse_train, 'test':sparse_test}
	else:
		return {'train':train, 'test':test}


def get_string (ftype, data, alphabet=features.DNA):
	"""Return StringFeatures.

	@param ftype Feature type, e.g. Real, Byte
	@param data Train/test data for feature creation
	@param alphabet Alphabet for feature creation
	@return Dict with StringFeatures train/test
	"""

	train=eval('features.String'+ftype+"Features(data['train'], alphabet)")
	test=eval('features.String'+ftype+"Features(data['test'], alphabet)")
	return {'train':train, 'test':test}


def get_string_complex (ftype, data, alphabet=features.DNA,
	order=WORDSTRING_ORDER, gap=WORDSTRING_GAP, reverse=WORDSTRING_REVERSE):
	"""Return complex StringFeatures.

	@param ftype Feature type, e.g. RealFeature, ByteFeature
	@param data Train/test data for feature creation
	@param alphabet Alphabet for feature creation
	@param order Order of the feature
	@param gap Gap of the feature
	@param reverse Is feature reverse?
	@return Dict with complex StringFeatures train/test
	"""

	feats={}

	charfeat=features.StringCharFeatures(data['train'], alphabet)
	feat=eval('features.String'+ftype+'Features(alphabet)')
	feat.obtain_from_char(charfeat, order-1, order, gap, reverse)
	feats['train']=feat

	charfeat=features.StringCharFeatures(data['test'], alphabet)
	feat=eval('features.String'+ftype+'Features(alphabet)')
	feat.obtain_from_char(charfeat, order-1, order, gap, reverse)
	feats['test']=feat

	if ftype=='Word' or ftype=='Ulong':
		name='Sort'+ftype+'String'
		return add_preproc(name, feats)
	else:
		return feats


def get_wd (data, order=WORDSTRING_ORDER):
	"""Return WDFeatures.

	@param data Train/test data for feature creation
	@param order Order of the feature
	@return Dict with WDFeatures train/test
	"""

	feats={}

	charfeat=features.StringCharFeatures(data['train'], features.DNA)
	bytefeat=features.StringByteFeatures(features.RAWDNA)
	bytefeat.obtain_from_char(charfeat, 0, 1, 0, False)
	feats['train']=features.WDFeatures(bytefeat, order, order)

	charfeat=features.StringCharFeatures(data['test'], features.DNA)
	bytefeat=features.StringByteFeatures(features.RAWDNA)
	bytefeat.obtain_from_char(charfeat, 0, 1, 0, False)
	feats['test']=features.WDFeatures(bytefeat, order, order)

	return feats


def add_preproc (name, feats, *args):
	"""Add a preprocessor to the given features.

	@param name Name of the preprocessor
	@param feats Features train/test
	@param *args Variable argument list of the preprocessor
	@return Dict with features having a preprocessor applied
	"""

	fun=eval('preproc.'+name)
	preproc=fun(*args)
	preproc.init(feats['train'])
	feats['train'].add_preprocessor(preproc)
	feats['train'].apply_preprocessor()
	feats['test'].add_preprocessor(preproc)
	feats['test'].apply_preprocessor()

	return feats




from sg import sg
from pylab import pcolor, scatter, contour, colorbar, show, imshow, connect
from numpy import min, max, where
import util

util.set_title('SVM Classification')

#sg('loglevel', 'ALL')
traindata=util.get_traindata()
labels=util.get_labels()
width=1.
size_cache=10

sg('set_features', 'TRAIN', traindata)
sg('set_labels', 'TRAIN', labels)
sg('set_kernel', 'GAUSSIAN', 'REAL', size_cache, width)
sg('new_classifier', 'LIBSVM')
sg('c', 100.)
sg('train_classifier')
[bias, alphas]=sg('get_svm')
#print bias
#print alphas

#print "objective: %f" % sg('get_svm_objective')

x, y=util.get_meshgrid(traindata)
testdata=util.get_testdata(x, y)
sg('set_features', 'TEST', testdata)
z=sg('classify')

z.resize((50,50))

#for non-smooth visualization
#pcolor(x, y, z, shading='flat')
#for smooth visualization
i=imshow(z,  origin='lower', extent=(
	1.2*min(traindata), 1.2*max(traindata), 1.2*min(traindata),
	1.2*max(traindata)))

negidx=where(labels==-1)[0]
posidx=where(labels==+1)[0]
scatter(traindata[0, negidx], traindata[1, negidx],
	s=20, marker='o', c='b', hold=True)
scatter(traindata[0, posidx], traindata[1, posidx],
	s=20, marker='o', c='r', hold=True)
contour(x, y, z, linewidths=1, colors='black', hold=True)
colorbar(i)
connect('key_press_event', util.quit)
show()



"""
This module contains code for computing
Position Oligomer Importance Matrices
"""

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import numpy
from numpy import ones


def compute_poims(svm, kernel, poimdegree, max_len):
    """For a trained SVM, compute Position Oligomer Importance Matrices"""

    distr = ones((max_len,4))/4 ;
    kernel.prepare_POIM2(distr)

    kernel.compute_POIM2(poimdegree, svm) ;
    poim = kernel.get_POIM2()
    kernel.cleanup_POIM2()

    (poim, max_poim, diff_poim) = reshape_normalize_contribs(poim, poimdegree, max_len)
    (poim_weightmass, poim_totalmass) = compute_weight_mass(poim, poimdegree, max_len)

    poim_totalmass=poim_totalmass/numpy.sum(poim_totalmass)

    return (poim, max_poim, diff_poim, poim_totalmass)


def compute_weight_mass(C, maxOrder, seqLen):

    mass=numpy.zeros((maxOrder, seqLen), numpy.double);
    total=numpy.zeros((1, seqLen), numpy.double);
    for i in xrange(0,maxOrder):
        mass[i,:] = sum(numpy.abs(C[i]))
    total = sum(mass);

    return (mass,total)

def getstringprobsMC(maxOrder,distrib,length, abcSize):

    pmatrix = []
    for k in xrange(0,maxOrder):
	pmatrix.append(ones(4^k,len))

        for l in xrange(0,len):
            for sigma in xrange(0, abcSize):
                prob = distrib(sigma,l);
                for k in xrange(0, maxOrder):
                    for relpos in xrange(0, min(k,l)):
                        vi = genindexvector_spos(k,sigma-1,relpos,abcSize);
                        pmatrix[k][vi,l-relpos+1] = pmatrix[k][vi,l-relpos+1]*prob;

    return pmatrix

def getV2_poimMC(u, strprobs, abcSize):
  VV = [];
  for k in xrange(0, len(u)):
      m = abcSize^k;
      VV.append( numpy.ones(4**(k+1),1)*mean(u[k]*strprobs[k] ) )

  return VV

def reshape_normalize_contribs(C, maxOrder, seqLen, opts={}):

    alphabetSize = 4;
    Contribs = [] ;
    l=0;
    for i in xrange(0, maxOrder):
        L = l + (alphabetSize**(i+1)) * seqLen;
        vec=C[l:L].copy() ;
        Contribs.append(vec.reshape( seqLen, alphabetSize**(i+1) ).T) ;
        l = L;

    assert( l == len(C) );

    if opts.has_key("distribution"):
        strprobs = getstringprobsMC(length(Contribs), opts["distribution"], seqLen, 4);
        MyV2 = getV2_poimMC(Contribs, strprobs, seqLen, 4);

        for i in xrange(0, maxOrder ):
            Contribs[i] = Contribs[i] -MyV2[i];

    if opts.has_key("background"):
        for i in xrange(0, maxOrder ):
            Contribs[i] = Contribs[i]*(opts["background"][i]!=0);

    maxContribs = numpy.zeros( (maxOrder, seqLen), numpy.double );
    maxp_org = numpy.zeros( (maxOrder, seqLen), numpy.double );
    maxp_str= numpy.zeros( (maxOrder, seqLen), numpy.int );
    for i in xrange(0, maxOrder ):
        con=numpy.abs(Contribs[i]) ;
	maxContribs[i,:] = numpy.max(con, axis=0)
	maxp_str[i,:] = numpy.argmax(con, axis=0)

    diffmaxContribs = numpy.zeros( (maxOrder, seqLen), numpy.double );

    for k in xrange(1, maxOrder ):
	numsy=4**(k+1);
	for l in  xrange(0, seqLen-k):
            km=maxp_str[k,l] ;
            A=numpy.abs(Contribs[k-1][numpy.floor(km/4),l]);
            B=numpy.abs(Contribs[k-1][numpy.mod(km,numsy/4),l+1]);
            #zA=numpy.mod(km,4)+1;
            #zB=numpy.floor(km/(numsy/4))+1;
            #correction=sum([A/distribution(zA, l+k-1), B/distribution(zB, l)]);
            correction=numpy.max([A, B]);
            diffmaxContribs[k,l] = maxContribs[k,l] - correction;

    return (Contribs, maxContribs, diffmaxContribs)




#!/usr/bin/env python
data = '../data/fm_train_real.dat'
parameter_list = [[data,20],[data,30]]

def converter_hessianlocallylinearembedding_modular (data_fname,k):
	try:
		from modshogun import RealFeatures, HessianLocallyLinearEmbedding, CSVFile

		features = RealFeatures(CSVFile(data))

		converter = HessianLocallyLinearEmbedding()
		converter.set_target_dim(1)
		converter.set_k(k)
		converter.apply(features)

		return features
	except ImportError:
		print('No Eigen3 available')

if __name__=='__main__':
	print('HessianLocallyLinearEmbedding')
	converter_hessianlocallylinearembedding_modular(*parameter_list[0])




# File   : $HeadURL$
# Version: $Id$

import gtk

class QuadrWidget(gtk.DrawingArea):

    # No SELF.CONNECT because we like to prevent the call of
    # gtk.DrawingArea.do_size_allocate()
    __gsignals__ = {"size-allocate": 'override'}

    def __init__(self):
        gtk.DrawingArea.__init__(self)

    def update(self):
        width = self.window.get_size()[0]
        height = self.window.get_size()[1]
        self.window.invalidate_rect(gtk.gdk.Rectangle(
                0, 0, width, height), False)
        #self.window.process_updates(False)

    def do_size_allocate(self, allocation):
        if allocation.width < allocation.height:
            allocation.y += (allocation.height-allocation.width)/2
            allocation.height = allocation.width
        elif allocation.width > allocation.height:
            allocation.x += (allocation.width-allocation.height)/2
            allocation.width = allocation.height

        gtk.DrawingArea.do_size_allocate(self, allocation)



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'

parameter_list = [[traindat,testdat,10,1.2,1.3],[traindat,testdat,10,1.2,1.3]]

def kernel_sigmoid_modular (train_fname=traindat,test_fname=testdat,size_cache=10,gamma=1.2,coef0=1.3):
	from modshogun import RealFeatures, SigmoidKernel, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	kernel=SigmoidKernel(feats_train, feats_train, size_cache, gamma, coef0)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('Sigmoid')
	kernel_sigmoid_modular(*parameter_list[0])



"""Common operations on train and test data"""

import sys
import md5
import string
import numpy
from numpy import random, round
from shogun.Features import BinaryLabels, RegressionLabels

NUM_VEC_TRAIN=11
NUM_VEC_TEST=17
LEN_SEQ=60
INIT_RANDOM=42
DIGITS_ROUND=7

_NUM_FEATS=11


def get_rand (dattype=numpy.double, num_feats=_NUM_FEATS, dim_square=False,
	max_train=sys.maxint, max_test=sys.maxint):
	"""Return random numbers.

	Return random numbers, either float or integer, in a dict with elements
	'train' and 'test'.

	@param dattype (numpy) data type of the random numbers
	@param num_feats number of features for in a train/test vector
	@param dim_square Dimensions of the square of random numbers, implying that number of features and vectors in both train/test are all the same.
	@param max_train Maximum value for data in train vectors
	@param max_test Maximum value for data in test vectors
	@return Dict which contains the random numbers
	"""


	if dim_square:
		num_feats=num_vec_train=num_vec_test=dim_square
	else:
		num_vec_train=NUM_VEC_TRAIN
		num_vec_test=NUM_VEC_TEST

	if dattype==numpy.double:
		return {
			'train':round(random.rand(num_feats, num_vec_train), DIGITS_ROUND),
			'test':round(random.rand(num_feats, num_vec_test), DIGITS_ROUND)
		}
	elif dattype==numpy.chararray:
		ord_a=ord('A')
		ord_z=ord('Z')
		rand_train=random.randint(ord_a, ord_z, num_feats*num_vec_train)
		rand_test=random.randint(ord_a, ord_z, num_feats*num_vec_test)
		return {
			'train': numpy.array(map(lambda x: chr(x), rand_train)).reshape(
				num_feats, num_vec_train),
			'test': numpy.array(map(lambda x: chr(x), rand_test)).reshape(
				num_feats, num_vec_test)
		}
	else:
		if dattype==numpy.ushort:
			maxval=2**16-1
			if max_train>maxval:
				max_train=maxval
			if max_test>maxval:
				max_test=maxval

		# randint does not understand arg dtype
		rand_train=random.randint(0, max_train, (num_feats, num_vec_train))
		rand_test=random.randint(0, max_test, (num_feats, num_vec_test))
		return {
			'train':rand_train.astype(dattype),
			'test':rand_test.astype(dattype)
		}


def get_clouds (num_clouds, num_feats=_NUM_FEATS):
	"""Return random float numbers organised, but scrambled, in clouds.

	The float numbers generated here are first created in number clouds, shifted from each other by a constant value. Then they are permutated to hide these clusters a bit when used in classifiers or clustering methods.
	It is a specialised case of random float number generation.

	@param num_clouds Number of clouds to generate
	@param num_features Number of features in each cloud
	@return Dict which contains the random numbers
	"""

	clouds={}

	data=[random.rand(num_feats, NUM_VEC_TRAIN)+x/2 for x in xrange(num_clouds)]
	clouds['train']=numpy.concatenate(data, axis=1)
	clouds['train']=numpy.array([random.permutation(x) for x in clouds['train']])
	clouds['train']=round(clouds['train'], DIGITS_ROUND)

	data=[random.rand(num_feats, NUM_VEC_TEST)+x/2 for x in xrange(num_clouds)]
	clouds['test']=numpy.concatenate(data, axis=1)
	clouds['test']=numpy.array([random.permutation(x) for x in clouds['test']])
	clouds['test']=round(clouds['test'], DIGITS_ROUND)

	return clouds


def get_cubes (num_train=4, num_test=8):
	"""Return cubes of with random emissions.

	Used by the Hidden-Markov-Model, it creates two
	seemingly random sequences of emissions of a 6-sided cube.

	@param num Number of hidden cubes
	@return Dict of tuples of emissions, representing a hidden cube
	"""

	leng=50
	rep=5
	weight=1


	sequence={'train':list(), 'test':list()}
	num={'train': num_train, 'test': num_test}

	for tgt in ('train', 'test'):
		for i in xrange(num[tgt]):
			# generate a sequence with characters 1-6 drawn from 3 loaded cubes
			loaded=[]
			for j in xrange(3):
				draw=[x*numpy.ones((1, numpy.ceil(leng*random.rand())), int)[0] \
					for x in xrange(1, 7)]
				loaded.append(random.permutation(numpy.concatenate(draw)))

			draws=[]
			for j in xrange(len(loaded)):
				ones=numpy.ones((1, numpy.ceil(rep*random.rand())), int)
				draws=numpy.concatenate((j*ones[0], draws))
			draws=random.permutation(draws)

			seq=[]
			for j in xrange(len(draws)):
				len_loaded=len(loaded[draws[j]])
				weighted=int(numpy.ceil(
					((1-weight)*random.rand()+weight)*len_loaded))
				perm=random.permutation(len_loaded)
				shuffled=[str(loaded[draws[j]][x]) for x in perm[:weighted]]
				seq=numpy.concatenate((seq, shuffled))

			sequence[tgt].append(''.join(seq))

	return sequence


def get_labels (num, ltype='twoclass'):
	"""Return labels used for classification.

	@param num Number of labels
	@param ltype Type of labels, either twoclass or series.
	@return Tuple to contain the labels as numbers in a tuple and labels as objects digestable for Shogun.
	"""

	labels=[]
	if ltype=='twoclass':
		labels.append(random.rand(num).round()*2-1)
		# essential to wrap in array(), will segfault sometimes otherwise
		labels.append(BinaryLabels(numpy.array(labels[0])))
	elif ltype=='series':
		labels.append([numpy.double(x) for x in xrange(num)])
		# essential to wrap in array(), will segfault sometimes otherwise
		labels.append(RegressionLabels(numpy.array(labels[0])))
	else:
		return [None, None]

	return labels


def get_dna (
	num_vec_train=NUM_VEC_TRAIN, num_vec_test=NUM_VEC_TEST, len_seq=LEN_SEQ):
	"""Return a random DNA sequence.

	@param num_vec_train number of training vectors
	@param num_vec_test number of test vectors
	@param len_seq sequence length
	@return Dict of tuples of DNA sequences.
	"""

	acgt=numpy.array(['A', 'C', 'G','T'])
	len_acgt=len(acgt)

	train=[
		''.join([
			acgt[numpy.floor(len_acgt*random.rand())] for j in xrange(len_seq)
		]) for i in xrange(num_vec_train)
	]

	test=[
		''.join([
			acgt[numpy.floor(len_acgt*random.rand())] for j in xrange(len_seq)
		]) for i in xrange(num_vec_test)
	]

	return {'train': train, 'test': test}



#!/usr/bin/env python
from tools.load import LoadMatrix
from numpy import random
lm=LoadMatrix()

ground_truth = lm.load_labels('../data/label_train_twoclass.dat')
random.seed(17)
predicted = random.randn(len(ground_truth))

parameter_list = [[ground_truth,predicted]]

def evaluation_rocevaluation_modular (ground_truth, predicted):
	from modshogun import BinaryLabels
	from modshogun import ROCEvaluation

	ground_truth_labels = BinaryLabels(ground_truth)
	predicted_labels = BinaryLabels(predicted)

	evaluator = ROCEvaluation()
	evaluator.evaluate(predicted_labels,ground_truth_labels)

	return evaluator.get_ROC(), evaluator.get_auROC()


if __name__=='__main__':
	print('ROCEvaluation')
	evaluation_rocevaluation_modular(*parameter_list[0])




#!/usr/bin/env python

from numpy import *
from pylab import plot, show, legend, fill_between, figure, subplot, title

def regression_gaussian_process_modelselection (n=100, n_test=100, \
		x_range=5, x_range_test=10, noise_var=0.4):

	from modshogun import RealFeatures, RegressionLabels
	from modshogun import GaussianKernel
	from modshogun import GradientModelSelection, ModelSelectionParameters
	from modshogun import GaussianLikelihood, ZeroMean, \
		ExactInferenceMethod, GaussianProcessRegression, GradientCriterion, \
		GradientEvaluation

	# easy regression data: one dimensional noisy sine wave
	X_train = random.rand(1,n)*x_range
	X_test = array([[float(i)/n_test*x_range_test for i in range(n_test)]])

	y_test = sin(X_test)
	y_train = sin(X_train)+random.randn(n)*noise_var

	# shogun representation
	labels = RegressionLabels(y_train[0])
	feats_train = RealFeatures(X_train)
	feats_test = RealFeatures(X_test)

	# GP specification
	kernel = GaussianKernel(10, 0.05)

	mean = ZeroMean()

	likelihood = GaussianLikelihood(0.8)

	inf = ExactInferenceMethod(kernel, feats_train, mean, labels, likelihood)
	inf.set_scale(2.5)

	gp = GaussianProcessRegression(inf)

	means = gp.get_mean_vector(feats_test)
	variances = gp.get_variance_vector(feats_test)

	# plot results
	figure()

	subplot(2, 1, 1)
	title('Initial parameter\'s values')

	plot(X_train[0], y_train[0], 'bx') # training observations

	plot(X_test[0], y_test[0], 'g-') # ground truth of test
	plot(X_test[0], means, 'r-') # mean predictions of test

	fill_between(X_test[0], means-1.96*sqrt(variances),
				 means+1.96*sqrt(variances), color='grey')

	legend(["training", "ground truth", "mean predictions"])

	# evaluate our inference method for its derivatives
	grad = GradientEvaluation(gp, feats_train, labels, GradientCriterion(), False)
	grad.set_function(inf)

	# handles all of the above structures in memory
	grad_search = GradientModelSelection(grad)

	# search for best parameters
	best_combination = grad_search.select_model(True)

	# outputs all result and information
	best_combination.apply_to_machine(gp)

	means = gp.get_mean_vector(feats_test)
	variances = gp.get_variance_vector(feats_test)

	# plot results
	subplot(2, 1, 2)
	title('Selected by gradient search parameter\'s values')

	plot(X_train[0], y_train[0], 'bx') # training observations

	plot(X_test[0], y_test[0], 'g-') # ground truth of test
	plot(X_test[0], means, 'r-') # mean predictions of test

	fill_between(X_test[0], means-1.96*sqrt(variances),
				 means+1.96*sqrt(variances), color='grey')

	legend(["training", "ground truth", "mean predictions"])

	show()

if __name__=='__main__':

	regression_gaussian_process_modelselection()



#!/usr/bin/env python

def load_compressed_features(fname_features):
	try:
		import gzip
		import numpy
	except ImportError:
		print 'Error importing gzip and/or numpy modules. Please, verify their installation.'
		import sys
		sys.exit(0)

	# load features from a gz compressed file
	file_features = gzip.GzipFile(fname_features)
	str_features = file_features.read()
	file_features.close()

	strlist_features = str_features.split('\n')[:-1] # all but last because the last line also has \n

	# the number of lines in the file is the number of vectors
	num_vectors = len(strlist_features)
	# the number of elements in a line is the number of features
	num_features = len(strlist_features[0].split())
	# memory pre-allocation for the feature matrix
	fm = numpy.zeros((num_vectors, num_features))

	# fill in feature matrix
	for i in xrange(num_vectors):
		try:
			fm[i,:] = map(numpy.float64, strlist_features[i].split())
		except ValuError:
			print 'All the vectors must have the same number of features.'
			import sys
			sys.exit(0)

	return fm

def metric_lmnn_statistics(k=3, fname_features='../../data/fm_train_multiclass_digits.dat.gz', fname_labels='../../data/label_train_multiclass_digits.dat'):
	try:
		from modshogun import LMNN, CSVFile, RealFeatures, MulticlassLabels, MSG_DEBUG
		import matplotlib.pyplot as pyplot
	except ImportError:
		print 'Error importing modshogun or other required modules. Please, verify their installation.'
		return

	features = RealFeatures(load_compressed_features(fname_features).T)
	labels = MulticlassLabels(CSVFile(fname_labels))

#	print 'number of examples = %d' % features.get_num_vectors()
#	print 'number of features = %d' % features.get_num_features()

	assert(features.get_num_vectors() == labels.get_num_labels())

	# train LMNN
	lmnn = LMNN(features, labels, k)
	lmnn.set_correction(100)
#	lmnn.io.set_loglevel(MSG_DEBUG)
	print 'Training LMNN, this will take about two minutes...'
	lmnn.train()
	print 'Training done!'

	# plot objective obtained during training
	statistics = lmnn.get_statistics()

	pyplot.plot(statistics.obj.get())
	pyplot.grid(True)
	pyplot.xlabel('Iterations')
	pyplot.ylabel('LMNN objective')
	pyplot.title('LMNN objective during training for the multiclass digits data set')

	pyplot.show()

if __name__=='__main__':
	print('LMNN objective')
	metric_lmnn_statistics()



#!/usr/bin/env python
traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
label_traindat = '../data/label_train_twoclass.dat'

parameter_list = [[traindat,testdat,label_traindat,0.9,1e-3],[traindat,testdat,label_traindat,0.8,1e-2]]

def classifier_liblinear_modular (train_fname, test_fname,
		label_fname, C, epsilon):

	from modshogun import RealFeatures, SparseRealFeatures, BinaryLabels
	from modshogun import LibLinear, L2R_L2LOSS_SVC_DUAL
	from modshogun import Math_init_random, CSVFile
	Math_init_random(17)

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))
	labels=BinaryLabels(CSVFile(label_fname))

	svm=LibLinear(C, feats_train, labels)
	svm.set_liblinear_solver_type(L2R_L2LOSS_SVC_DUAL)
	svm.set_epsilon(epsilon)
	svm.set_bias_enabled(True)
	svm.train()

	predictions = svm.apply(feats_test)
	return predictions, svm, predictions.get_labels()

if __name__=='__main__':
	print('LibLinear')
	classifier_liblinear_modular(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
parameter_list=[[traindna,testdna,10,20],
		[traindna,testdna,11,21]]

def kernel_weighteddegreepositonstring (fm_train_dna=traindna,fm_test_dna=testdna,
			size_cache=10,degree=20):

	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('set_kernel', 'WEIGHTEDDEGREEPOS', 'CHAR', size_cache, degree)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('WeightedDegreePositionString')
	kernel_weighteddegreepositonstring(*parameter_list[0])



#!/usr/bin/env python

from numpy import *
from scipy.io import mmread

# Loading an example sparse matrix of dimension 479x479, real, unsymmetric
mtx=mmread('../../../data/logdet/west0479.mtx.gz')

parameter_list=[[mtx,100,60,1]]

def mathematics_logdet (matrix=mtx,max_iter_eig=1000,max_iter_lin=1000,num_samples=1):

	from scipy.sparse import eye

	# Create a Hermitian sparse matrix
	rows=matrix.shape[0]
	cols=matrix.shape[1]
	A=matrix.transpose()*matrix+eye(rows, cols)

	from scipy.sparse import csc_matrix

	try:
		from shogun.Mathematics import RealSparseMatrixOperator
		from shogun.Mathematics import LanczosEigenSolver
		from shogun.Mathematics import CGMShiftedFamilySolver
		from shogun.Mathematics import LogRationalApproximationCGM
		from shogun.Mathematics import ProbingSampler
		from shogun.Mathematics import LogDetEstimator
		from shogun.Mathematics import Statistics
		from shogun.Library import SerialComputationEngine

		# creating the linear operator, eigen-solver
		op=RealSparseMatrixOperator(A.tocsc())

		eig_solver=LanczosEigenSolver(op)

		# we can set the iteration limit high for poorly conditioned matrices
		eig_solver.set_max_iteration_limit(max_iter_eig)

		# alternatively, if the matrix is small, we can compute eigenvalues externally
		# and set min/max eigenvalues into the eigensolver
		# from scipy.sparse.linalg import eigsh

		# eigenvalues=eigsh(A, rows-1)
		# eig_solver.set_min_eigenvalue(eigenvalues[0][0])
		# eig_solver.set_max_eigenvalue(eigenvalues[0][-1])

		# create the shifted-family linear solver which solves for all the shifts
		# using as many matrix-vector products as one shift in CG iterations
		lin_solver=CGMShiftedFamilySolver()
		lin_solver.set_iteration_limit(max_iter_lin)

		# computation engine
		engine=SerialComputationEngine()

		# set the desired accuracy tighter to obtain better results
		# this determines the number of contour points in conformal mapping of
		# the rational approximation of the Cauchy's integral of f(A)*s, f=log
		desired_accuracy=1E-5

		# creating the log-linear-operator function
		op_func=LogRationalApproximationCGM(op, engine, eig_solver, lin_solver,\
			desired_accuracy)

		# set the trace sampler to be probing sampler, in which samples are obtained
		# by greedy graph coloring of the power of sparse matrix (default is power=1,
		# 2-distance coloring)
		trace_sampler=ProbingSampler(op)

		# estimating log-det
		log_det_estimator=LogDetEstimator(trace_sampler, op_func, engine)

		# set the number of samples as required
		estimates=log_det_estimator.sample(num_samples)

		estimated_logdet=sum(estimates)/len(estimates)
		actual_logdet=Statistics.log_det(A)

		print actual_logdet, estimated_logdet

		return estimates

	except ImportError:
		print 'One or many of the dependencies (Eigen3/LaPack/ColPack) not found!'

if __name__=='__main__':
	print('LogDetEstimator')
	mathematics_logdet (*parameter_list[0])



#!/usr/bin/env python

import numpy as np
import matplotlib.pyplot as plt
from pylab import show, imshow

def simulate_data (n,p):
	from modshogun import SparseInverseCovariance
	import numpy as np

	#create a random pxp covariance matrix
	cov = np.random.normal(size=(p,p))

	#generate data set with multivariate Gaussian distribution
	mean = [0] * p
	data = np.random.multivariate_normal(mean, cov, n)

	return data

def inverse_covariance (data,lc):
	from modshogun import SparseInverseCovariance
	from numpy import dot

	sic = SparseInverseCovariance()

	#by default cov() expects each row to represent a variable, with observations in the columns
	cov = np.cov(data.T)

	max_cov = cov.max()
	min_cov = cov.min()

	#compute inverse conariance matrix
	Si = sic.estimate(cov,lc)

	return Si


def draw_graph(sic, subplot):
	import numpy as np
	import networkx as nx

	#create list of edges
	#an egde means there is a dependency between variables
	#0 value in sic matrix mean independent variables given all the other variables
	p = sic.shape[0]
	X, Y = np.meshgrid(range(p), range(p))
	graph = np.array((X[sic != 0], Y[sic != 0])).T

	# extract nodes from graph
	nodes = set([n1 for n1, n2 in graph] + [n2 for n1, n2 in graph])

    # create networkx graph
	G=nx.Graph()

    # add nodes
	for node in nodes:
		G.add_node(node)

    # add edges
	for edge in graph:
		G.add_edge(edge[0], edge[1])

    # draw graph
	nx.draw(G, ax=subplot)

    # show graph
	return graph


if __name__=='__main__':


	#edit here for your own simulation
	num_observations = 100
	num_variables = 11
	penalties = [0.00001, 0.05, 0.1, 0.5, 1, 2]

	columns = len(penalties)

	#plot the heat map and the graphs of dependency between variables
	#for different penaltiy values
	f, axarr = plt.subplots(2, columns)
	f.suptitle('Inverse Covariance Estimation\nfor ' +str(num_variables)+' variables and '+str(num_observations)+' observations', fontsize=20)

	data = simulate_data (num_observations, num_variables)
	print data.shape

	column = -1;
	for p in penalties:
		column = column + 1

		sic = inverse_covariance (data,p)

		i = axarr[0, column].imshow(sic, cmap="hot", interpolation='nearest')
		axarr[0, column].set_title('penalty='+str(p), fontsize=10)
		graph = draw_graph(sic, plt.subplot(2, columns, column + columns + 1))
		axarr[1, column].set_title(str((len(graph) - num_variables)/2) + ' depedences', fontsize=10)


	f.subplots_adjust(right=0.8)
	cbar_ax = f.add_axes([0.85, 0.15, 0.05, 0.7])
	f.colorbar(i, cax=cbar_ax)

	show();





"""
This module contains code to parse the input arguments to the command line:
- easysvm.py
- datagen.py
"""

#############################################################################################
#                                                                                           #
#    This program is free software; you can redistribute it and/or modify                   #
#    it under the terms of the GNU General Public License as published by                   #
#    the Free Software Foundation; either version 3 of the License, or                      #
#    (at your option) any later version.                                                    #
#                                                                                           #
#    This program is distributed in the hope that it will be useful,                        #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of                         #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                           #
#    GNU General Public License for more details.                                           #
#                                                                                           #
#    You should have received a copy of the GNU General Public License                      #
#    along with this program; if not, see http://www.gnu.org/licenses                       #
#    or write to the Free Software Foundation, Inc., 51 Franklin Street,                    #
#    Fifth Floor, Boston, MA 02110-1301  USA                                                #
#                                                                                           #
#############################################################################################

import datafuncs
import sys

################################################################################
# basic types

def parse_range(str):
    list=str.split("-")

    if len(list)==1:
        return (int(list[0]), int(list[0]))
    if len(list)==2:
        return (int(list[0]), int(list[1]))
    sys.stderr.write("Cannot parse range '%s'\n" %str)
    sys.exit(-1)


def parse_float_list(str):
    list=str.split(",")
    float_list=[] ;
    for elem in list:
        float_list.append(float(elem))
    return float_list


def parse_int_list(str):
    list=str.split(",")
    int_list=[] ;
    for elem in list:
        int_list.append(int(elem))
    return int_list


################################################################################
# input files

def parse_input_file_train(kernelname, argv):
    """Parse the input and output file names"""

    if len(argv)<2 or (argv[0]=="fasta" and len(argv)<3) or (argv[0]!='fasta' and argv[0]!='arff'):
        sys.stderr.write("data usage: arff <train.arff>\n        or: fasta <train_pos.fa> <train_neg.fa>\n")
        sys.exit(-1)

    if argv[0] == 'fasta':
        datafilenamepos = argv[1]
        datafilenameneg = argv[2]
        (examples, labels) = datafuncs.fastaread(datafilenamepos, datafilenameneg)
        argv_rest=argv[3:]
    elif argv[0] == 'arff':
        datafilename = argv[1]
        (examples, labels) = datafuncs.arffread(kernelname, datafilename)
        argv_rest=argv[2:]
    else:
        print 'Error in parse_input_file'

    return (examples,labels,argv_rest)


def parse_input_file_train_test(kernelname, argv):
    """Parse the input and output file names"""

    if len(argv)<3 or (argv[0]=="fasta" and len(argv)<4) or (argv[0]!='fasta' and argv[0]!='arff'):
        sys.stderr.write("data usage: arff <train.arff> <test.arff>\n        or: fasta <train_pos.fa> <train_neg.fa> <test.fa>\n")
        sys.exit(-1)

    if argv[0] == 'fasta':
        datafilenamepos = argv[1]
        datafilenameneg = argv[2]
        datafilenametest = argv[3]
        (trainex, trainlab) = datafuncs.fastaread(datafilenamepos, datafilenameneg)
        (testex, testlab) = datafuncs.fastaread(datafilenametest)
        argv_rest=argv[4:]
    elif argv[0] == 'arff':
        datafilename = argv[1]
        datafilenametest = argv[2]
        (trainex, trainlab) = datafuncs.arffread(kernelname, datafilename)
        (testex, testlab) = datafuncs.arffread(kernelname, datafilenametest)
        argv_rest=argv[3:]
    else:
        print 'Error in parse_input_file'

    return (trainex,trainlab,testex,argv_rest)

################################################################################
# prediction file

def parse_prediction_file(fname):
    outputs=[]
    splitassignments=[]

    f = open(fname)
    str=f.read()
    lines = str.split('\n')
    num=0
    for line in lines:
        if len(line)>0 and line[0] != '#':
            elems=line.split('\t')
            assert(len(elems)>1)
            assert(int(elems[0]) == num)
            num+=1
            if len(elems)==2:
                outputs.append(float(elems[1]))
            else:
                assert(len(elems)==3)
                outputs.append(float(elems[1]))
                splitassignments.append(float(elems[2]))
    f.close()
    if len(splitassignments)==0:
        splitassignments = None

    return (outputs, splitassignments)

################################################################################
# kernel parameters

def parse_kernel_param(argv, allow_modelsel_params):
    """Parse the arguments for a particular kernel"""

    if len(argv)<1:
        sys.stderr.write("kernel usage: <kernelname> [<parameters>]\n")
        sys.exit(-1)

    kernelname = argv[0]
    kparam = {}
    kparam["name"]=kernelname
    kparam["modelsel_name"]=None
    kparam["modelsel_params"]=None

    if kernelname == 'gauss':
        if len(argv)<2:
            sys.stderr.write("kernel usage: gauss <width>\n")
            sys.exit(-1)
        if allow_modelsel_params:
            kparam['width'] = None
            kparam["modelsel_name"]="width"
            kparam["modelsel_params"]=parse_float_list(argv[1])
        else:
            kparam['width'] = float(argv[1])
        argv_rest=argv[2:]
    elif kernelname == 'linear':
	kparam['scale']=1
        # no parameters
        argv_rest=argv[1:]
    elif kernelname == 'poly':
        if len(argv)<4:
            sys.stderr.write("kernel usage: poly <degree> <true|false> <true|false>\n")
            sys.exit(-1)
        if allow_modelsel_params:
            kparam['degree'] = None
            kparam["modelsel_name"]="degree"
            kparam["modelsel_params"]=parse_int_list(argv[1])
        else:
            kparam['degree'] = int(argv[1])
        kparam['inhomogene'] = (argv[2] == 'true')
        kparam['normal'] = (argv[3] == 'true')
        argv_rest=argv[4:]
    elif kernelname == 'wd':
        if len(argv)<3:
            sys.stderr.write("kernel usage: wd <degree> <shift>\n")
            sys.exit(-1)
        if allow_modelsel_params:
            kparam['degree'] = None
            kparam["modelsel_name"]="degree"
            kparam["modelsel_params"]=parse_int_list(argv[1])
        else:
            kparam['degree'] = int(argv[1])
        if allow_modelsel_params and len(kparam["modelsel_params"])==1:
            kparam['degree'] = kparam["modelsel_params"][0]
            kparam['shift'] = None
            kparam["modelsel_name"] = "shift"
            kparam["modelsel_params"]=parse_int_list(argv[2])
        else:
            kparam['shift'] = int(argv[2])
        argv_rest=argv[3:]
    elif kernelname == 'spec':
        if len(argv)<2:
            sys.stderr.write("kernel usage: spec <degree>\n")
            sys.exit(-1)
        if allow_modelsel_params:
            kparam['degree'] = None
            kparam["modelsel_name"]="degree"
            kparam["modelsel_params"]=parse_int_list(argv[1])
        else:
            kparam['degree'] = int(argv[1])
        argv_rest=argv[2:]
    elif kernelname == 'localalign':
        # no parameters
        argv_rest=argv[1:]
    elif kernelname == 'localimprove':
        if len(argv)<4:
            sys.stderr.write("kernel usage: localimprove <length> <indegree> <outdegree>\n")
            sys.exit(-1)
        kparam['length'] = int(argv[1])
        if allow_modelsel_params:
            kparam['width'] = None
            kparam["modelsel_name"]="indeg"
            kparam["modelsel_params"]=parse_int_list(argv[2])
        else:
            kparam['indeg'] = int(argv[2])
        kparam['outdeg'] = int(argv[3])
        argv_rest=argv[4:]
    else:
        sys.stderr.write( 'Unknown kernel name %s in parse_kernel_param\n' % kernelname )
        sys.exit(-1)

    return kernelname,kparam,argv_rest




#!/usr/bin/env python
from numpy import array
from numpy.random import seed, rand
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')
label_traindat = lm.load_labels('../data/label_train_twoclass.dat')

parameter_list = [[traindat,testdat,label_traindat]]

def transfer_multitask_leastsquares_regression (fm_train=traindat,fm_test=testdat,label_train=label_traindat):

	from modshogun import RegressionLabels, RealFeatures, Task, TaskGroup, MultitaskLeastSquaresRegression

	features = RealFeatures(traindat)
	labels = RegressionLabels(label_train)

	n_vectors = features.get_num_vectors()
	task_one = Task(0,n_vectors//2)
	task_two = Task(n_vectors//2,n_vectors)
	task_group = TaskGroup()
	task_group.append_task(task_one)
	task_group.append_task(task_two)

	mtlsr = MultitaskLeastSquaresRegression(0.1,features,labels,task_group)
	mtlsr.set_regularization(1) # use regularization ratio
	mtlsr.set_tolerance(1e-2) # use 1e-2 tolerance
	mtlsr.train()
	mtlsr.set_current_task(0)
	out = mtlsr.apply_regression().get_labels()
	return out

if __name__=='__main__':
	print('TransferMultitaskLeastSquaresRegression')
	transfer_multitask_leastsquares_regression(*parameter_list[0])



#!/usr/bin/env python
testdat = '../data/fm_train_real.dat'
traindat = '../data/fm_test_real.dat'

parameter_list=[[traindat,testdat,1.7],[traindat,testdat,1.8]]

def kernel_distance_modular (train_fname=traindat,test_fname=testdat,width=1.7):
	from modshogun import RealFeatures, DistanceKernel, EuclideanDistance, CSVFile

	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))

	distance=EuclideanDistance()
	kernel=DistanceKernel(feats_train, feats_test, width, distance)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('Distance')
	kernel_distance_modular(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
from numpy import where
lm=LoadMatrix()

traindat = lm.load_dna('../data/fm_train_dna.dat')
testdat = lm.load_dna('../data/fm_test_dna.dat')
label_traindat = lm.load_labels('../data/label_train_dna.dat')
parameter_list = [[traindat,testdat,label_traindat,1,4,1e-1,1,0,False,[1,False,True]],[traindat,testdat,label_traindat,3,4,1e-1,1,0,False,[1,False,True]]]

fm_hmm_pos=[ traindat[i] for i in where([label_traindat==1])[1] ]
fm_hmm_neg=[ traindat[i] for i in where([label_traindat==-1])[1] ]

def kernel_fisher_modular (fm_train_dna=traindat, fm_test_dna=testdat,
		label_train_dna=label_traindat,
		N=1,M=4,pseudo=1e-1,order=1,gap=0,reverse=False,
		kargs=[1,False,True]):

	from modshogun import StringCharFeatures, StringWordFeatures, FKFeatures, DNA
	from modshogun import PolyKernel
	from modshogun import HMM, BW_NORMAL#, MSG_DEBUG

	# train HMM for positive class
	charfeat=StringCharFeatures(fm_hmm_pos, DNA)
	#charfeat.io.set_loglevel(MSG_DEBUG)
	hmm_pos_train=StringWordFeatures(charfeat.get_alphabet())
	hmm_pos_train.obtain_from_char(charfeat, order-1, order, gap, reverse)
	pos=HMM(hmm_pos_train, N, M, pseudo)
	pos.baum_welch_viterbi_train(BW_NORMAL)

	# train HMM for negative class
	charfeat=StringCharFeatures(fm_hmm_neg, DNA)
	hmm_neg_train=StringWordFeatures(charfeat.get_alphabet())
	hmm_neg_train.obtain_from_char(charfeat, order-1, order, gap, reverse)
	neg=HMM(hmm_neg_train, N, M, pseudo)
	neg.baum_welch_viterbi_train(BW_NORMAL)

	# Kernel training data
	charfeat=StringCharFeatures(fm_train_dna, DNA)
	wordfeats_train=StringWordFeatures(charfeat.get_alphabet())
	wordfeats_train.obtain_from_char(charfeat, order-1, order, gap, reverse)

	# Kernel testing data
	charfeat=StringCharFeatures(fm_test_dna, DNA)
	wordfeats_test=StringWordFeatures(charfeat.get_alphabet())
	wordfeats_test.obtain_from_char(charfeat, order-1, order, gap, reverse)

	# get kernel on training data
	pos.set_observations(wordfeats_train)
	neg.set_observations(wordfeats_train)
	feats_train=FKFeatures(10, pos, neg)
	feats_train.set_opt_a(-1) #estimate prior
	kernel=PolyKernel(feats_train, feats_train, *kargs)
	km_train=kernel.get_kernel_matrix()

	# get kernel on testing data
	pos_clone=HMM(pos)
	neg_clone=HMM(neg)
	pos_clone.set_observations(wordfeats_test)
	neg_clone.set_observations(wordfeats_test)
	feats_test=FKFeatures(10, pos_clone, neg_clone)
	feats_test.set_a(feats_train.get_a()) #use prior from training data
	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print("Fisher Kernel")
	kernel_fisher_modular(*parameter_list[0])



#!/usr/bin/env python
parameter_list=[[]]

def features_dense_io_modular():
	from modshogun import RealFeatures, CSVFile
	feats=RealFeatures()
	f=CSVFile("../data/fm_train_real.dat","r")
	f.set_delimiter(" ")
	feats.load(f)
	return feats

if __name__=='__main__':
	print('Dense Real Features IO')
	features_dense_io_modular(*parameter_list[0])



#!/usr/bin/env python

strings=['example document 1','example document 2','example document 3','example document 4']

parameter_list=[[strings]]

def converter_hasheddoc_modular(strings):
	from modshogun import SparseRealFeatures, RAWBYTE, StringCharFeatures, Features, HashedDocDotFeatures
	from modshogun import NGramTokenizer
	from modshogun import HashedDocConverter
	from numpy import array

	#create string features
	f=StringCharFeatures(strings, RAWBYTE)

	#set the number of bits of the target dimension
	#means a dim of size 2^5=32
	num_bits=5

	#create the ngram tokenizer of size 8 to parse the strings
	tokenizer=NGramTokenizer(8)

	#normalize results
	normalize=True

	#create converter
	converter=HashedDocConverter(tokenizer, num_bits, normalize)

	converted_feats=converter.apply(f)

	#should expect 32
	#print('Converted features\' space dimensionality is', converted_feats.get_dim_feature_space())

	#print('Self dot product of string 0 with converted feats:', converted_feats.dot(0, converted_feats, 0))

	hashed_feats=HashedDocDotFeatures(num_bits, f, tokenizer, normalize)

	#print('Hashed features\' space dimensionality is', hashed_feats.get_dim_feature_space())

	#print('Self dot product of string 0 with hashed feats:', hashed_feats.dot(0, hashed_feats, 0))

	return converted_feats

if __name__=='__main__':
	print('HashedDocConverter')
	converter_hasheddoc_modular(*parameter_list[0])





#!/usr/bin/env python
#
# This program is free software you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation either version 3 of the License, or
# (at your option) any later version.
#
# Written (C) 2012-2013 Heiko Strathmann
#

from numpy import array
from numpy import random

from modshogun import CrossValidation, CrossValidationResult
from modshogun import ContingencyTableEvaluation, ACCURACY
from modshogun import StratifiedCrossValidationSplitting
from modshogun import BinaryLabels
from modshogun import RealFeatures
from modshogun import GaussianKernel, PowerKernel
from modshogun import LibSVM
from modshogun import MinkowskiMetric
from modshogun import GridSearchModelSelection
from modshogun import ModelSelectionParameters, R_EXP, R_LINEAR
from modshogun import ParameterCombination
from modshogun import Math

def create_param_tree():
	root=ModelSelectionParameters()

	c1=ModelSelectionParameters("C1")
	root.append_child(c1)
	c1.build_values(-1.0, 1.0, R_EXP)

	c2=ModelSelectionParameters("C2")
	root.append_child(c2)
	c2.build_values(-1.0, 1.0, R_EXP)

	gaussian_kernel=GaussianKernel()

	# print all parameter available for modelselection
	# Dont worry if yours is not included, simply write to the mailing list
	#gaussian_kernel.print_modsel_params()

	param_gaussian_kernel=ModelSelectionParameters("kernel", gaussian_kernel)
	gaussian_kernel_width=ModelSelectionParameters("width")
	gaussian_kernel_width.build_values(-1.0, 1.0, R_EXP, 1.0, 2.0)
	param_gaussian_kernel.append_child(gaussian_kernel_width)
	root.append_child(param_gaussian_kernel)

	power_kernel=PowerKernel()

	# print all parameter available for modelselection
	# Dont worry if yours is not included, simply write to the mailing list
	#power_kernel.print_modsel_params()

	param_power_kernel=ModelSelectionParameters("kernel", power_kernel)
	root.append_child(param_power_kernel)

	param_power_kernel_degree=ModelSelectionParameters("degree")
	param_power_kernel_degree.build_values(1.0, 2.0, R_LINEAR)
	param_power_kernel.append_child(param_power_kernel_degree)

	metric=MinkowskiMetric(10)

	# print all parameter available for modelselection
	# Dont worry if yours is not included, simply write to the mailing list
	#metric.print_modsel_params()

	param_power_kernel_metric1=ModelSelectionParameters("distance", metric)

	param_power_kernel.append_child(param_power_kernel_metric1)

	param_power_kernel_metric1_k=ModelSelectionParameters("k")
	param_power_kernel_metric1_k.build_values(1.0, 2.0, R_LINEAR)
	param_power_kernel_metric1.append_child(param_power_kernel_metric1_k)

	return root

parameter_list = [[3,20,3]]

def modelselection_grid_search_kernel (num_subsets, num_vectors, dim_vectors):
	# init seed for reproducability
	Math.init_random(1)
	random.seed(1);

	# create some (non-sense) data
	matrix=random.rand(dim_vectors, num_vectors)

	# create num_feautres 2-dimensional vectors
	features=RealFeatures()
	features.set_feature_matrix(matrix)

	# create labels, two classes
	labels=BinaryLabels(num_vectors)
	for i in range(num_vectors):
		labels.set_label(i, 1 if i%2==0 else -1)

	# create svm
	classifier=LibSVM()

	# splitting strategy
	splitting_strategy=StratifiedCrossValidationSplitting(labels, num_subsets)

	# accuracy evaluation
	evaluation_criterion=ContingencyTableEvaluation(ACCURACY)

	# cross validation class for evaluation in model selection
	cross=CrossValidation(classifier, features, labels, splitting_strategy, evaluation_criterion)
	cross.set_num_runs(1)

	# print all parameter available for modelselection
	# Dont worry if yours is not included, simply write to the mailing list
	#classifier.print_modsel_params()

	# model parameter selection
	param_tree=create_param_tree()
	#param_tree.print_tree()

	grid_search=GridSearchModelSelection(cross, param_tree)

	print_state=False
	best_combination=grid_search.select_model(print_state)
	#print("best parameter(s):")
	#best_combination.print_tree()

	best_combination.apply_to_machine(classifier)

	# larger number of runs to have tighter confidence intervals
	cross.set_num_runs(10)
	cross.set_conf_int_alpha(0.01)
	result=cross.evaluate()
	casted=CrossValidationResult.obtain_from_generic(result);
	#print "result mean:", casted.mean

	return classifier,result,casted.mean

if __name__=='__main__':
	print('ModelselectionGridSearchKernel')
	modelselection_grid_search_kernel(*parameter_list[0])



from pylab import figure,scatter,contour,show,legend,connect
from numpy import array, append, arange, reshape, empty, exp
from modshogun import Gaussian, GMM
from modshogun import RealFeatures
import util

util.set_title('SMEM for 2d GMM example')

#set the parameters
max_iter=100
max_cand=5
min_cov=1e-9
max_em_iter=1000
min_change=1e-9
cov_type=0

#setup the real GMM
real_gmm=GMM(3)

real_gmm.set_nth_mean(array([2.0, 2.0]), 0)
real_gmm.set_nth_mean(array([-2.0, -2.0]), 1)
real_gmm.set_nth_mean(array([2.0, -2.0]), 2)

real_gmm.set_nth_cov(array([[1.0, 0.2],[0.2, 0.5]]), 0)
real_gmm.set_nth_cov(array([[0.2, 0.1],[0.1, 0.5]]), 1)
real_gmm.set_nth_cov(array([[0.3, -0.2],[-0.2, 0.8]]), 2)

real_gmm.set_coef(array([0.3, 0.4, 0.3]))

#generate training set from real GMM
generated=array([real_gmm.sample()])
for i in range(199):
    generated=append(generated, array([real_gmm.sample()]), axis=0)

generated=generated.transpose()
feat_train=RealFeatures(generated)

#train GMM using SMEM and print log-likelihood
est_smem_gmm=GMM(3, cov_type)
est_smem_gmm.train(feat_train)

est_smem_gmm.set_nth_mean(array([2.0, 0.0]), 0)
est_smem_gmm.set_nth_mean(array([-2.0, -2.0]), 1)
est_smem_gmm.set_nth_mean(array([-3.0, -3.0]), 2)

est_smem_gmm.set_nth_cov(array([[1.0, 0.0],[0.0, 1.0]]), 0)
est_smem_gmm.set_nth_cov(array([[1.0, 0.0],[0.0, 1.0]]), 1)
est_smem_gmm.set_nth_cov(array([[1.0, 0.0],[0.0, 1.0]]), 2)

est_smem_gmm.set_coef(array([0.3333, 0.3333, 0.3334]))

print est_smem_gmm.train_smem(max_iter, max_cand, min_cov, max_em_iter, min_change)

#train GMM using EM and bad initial conditions and print log-likelihood
est_em_gmm=GMM(3, cov_type)
est_em_gmm.train(feat_train)

est_em_gmm.set_nth_mean(array([2.0, 0.0]), 0)
est_em_gmm.set_nth_mean(array([-2.0, -2.0]), 1)
est_em_gmm.set_nth_mean(array([-3.0, -3.0]), 2)

est_em_gmm.set_nth_cov(array([[1.0, 0.0],[0.0, 1.0]]), 0)
est_em_gmm.set_nth_cov(array([[1.0, 0.0],[0.0, 1.0]]), 1)
est_em_gmm.set_nth_cov(array([[1.0, 0.0],[0.0, 1.0]]), 2)

est_em_gmm.set_coef(array([0.3333, 0.3333, 0.3334]))

print est_em_gmm.train_em(min_cov, max_em_iter, min_change)

#plot real GMM, data and both estimated GMMs
min_x_gen=min(min(generated[[0]]))-0.1
max_x_gen=max(max(generated[[0]]))+0.1
min_y_gen=min(min(generated[[1]]))-0.1
max_y_gen=max(max(generated[[1]]))+0.1

plot_real=empty(0)
plot_est_smem=empty(0)
plot_est_em=empty(0)

for i in arange(min_x_gen, max_x_gen, 0.05):
    for j in arange(min_y_gen, max_y_gen, 0.05):
        plot_real=append(plot_real, array([exp(real_gmm.cluster(array([i, j]))[3])]))
        plot_est_smem=append(plot_est_smem, array([exp(est_smem_gmm.cluster(array([i, j]))[3])]))
        plot_est_em=append(plot_est_em, array([exp(est_em_gmm.cluster(array([i, j]))[3])]))

plot_real=reshape(plot_real, (arange(min_x_gen, max_x_gen, 0.05).shape[0], arange(min_y_gen, max_y_gen, 0.05).shape[0]))
plot_est_smem=reshape(plot_est_smem, (arange(min_x_gen, max_x_gen, 0.05).shape[0], arange(min_y_gen, max_y_gen, 0.05).shape[0]))
plot_est_em=reshape(plot_est_em, (arange(min_x_gen, max_x_gen, 0.05).shape[0], arange(min_y_gen, max_y_gen, 0.05).shape[0]))

real_plot=contour(arange(min_x_gen, max_x_gen, 0.05), arange(min_y_gen, max_y_gen, 0.05), plot_real.transpose(), colors="b")
est_smem_plot=contour(arange(min_x_gen, max_x_gen, 0.05), arange(min_y_gen, max_y_gen, 0.05), plot_est_smem.transpose(), colors="r")
est_em_plot=contour(arange(min_x_gen, max_x_gen, 0.05), arange(min_y_gen, max_y_gen, 0.05), plot_est_em.transpose(), colors="g")
real_scatter=scatter(generated[[0]], generated[[1]], c="gray")
legend((real_plot.collections[0], est_em_plot.collections[0], est_smem_plot.collections[0]), ("Real GMM", "Estimated EM GMM", "Estimated SMEM GMM"))

connect('key_press_event', util.quit)
show()



#!/usr/bin/env python
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2012 Heiko Strathmann
# Copyright (C) 2012 Berlin Institute of Technology and Max-Planck-Society
#

from numpy.random import randn, seed
from numpy import *

# generate some overlapping training vectors
seed(1)
num_vectors=7
vec_distance=1
traindat=concatenate((randn(2,num_vectors)-vec_distance,
	randn(2,num_vectors)+vec_distance), axis=1)
label_traindat=concatenate((zeros(num_vectors), ones(num_vectors)));

parameter_list = [[traindat,label_traindat]]

def evaluation_cross_validation_multiclass_storage (traindat=traindat, label_traindat=label_traindat):
    from modshogun import CrossValidation, CrossValidationResult
    from modshogun import CrossValidationPrintOutput
    from modshogun import CrossValidationMKLStorage, CrossValidationMulticlassStorage
    from modshogun import MulticlassAccuracy, F1Measure
    from modshogun import StratifiedCrossValidationSplitting
    from modshogun import MulticlassLabels
    from modshogun import RealFeatures, CombinedFeatures
    from modshogun import GaussianKernel, CombinedKernel
    from modshogun import MKLMulticlass
    from modshogun import Statistics, MSG_DEBUG, Math

    Math.init_random(1)

    # training data, combined features all on same data
    features=RealFeatures(traindat)
    comb_features=CombinedFeatures()
    comb_features.append_feature_obj(features)
    comb_features.append_feature_obj(features)
    comb_features.append_feature_obj(features)
    labels=MulticlassLabels(label_traindat)

    # kernel, different Gaussians combined
    kernel=CombinedKernel()
    kernel.append_kernel(GaussianKernel(10, 0.1))
    kernel.append_kernel(GaussianKernel(10, 1))
    kernel.append_kernel(GaussianKernel(10, 2))

    # create mkl using libsvm, due to a mem-bug, interleaved is not possible
    svm=MKLMulticlass(1.0,kernel,labels);
    svm.set_kernel(kernel);

    # splitting strategy for 5 fold cross-validation (for classification its better
    # to use "StratifiedCrossValidation", but the standard
    # "StratifiedCrossValidationSplitting" is also available
    splitting_strategy=StratifiedCrossValidationSplitting(labels, 3)

    # evaluation method
    evaluation_criterium=MulticlassAccuracy()

    # cross-validation instance
    cross_validation=CrossValidation(svm, comb_features, labels,
        splitting_strategy, evaluation_criterium)
    cross_validation.set_autolock(False)

    # append cross vlaidation output classes
    #cross_validation.add_cross_validation_output(CrossValidationPrintOutput())
    #mkl_storage=CrossValidationMKLStorage()
    #cross_validation.add_cross_validation_output(mkl_storage)
    multiclass_storage=CrossValidationMulticlassStorage()
    multiclass_storage.append_binary_evaluation(F1Measure())
    cross_validation.add_cross_validation_output(multiclass_storage)
    cross_validation.set_num_runs(3)

    # perform cross-validation
    result=cross_validation.evaluate()

    roc_0_0_0 = multiclass_storage.get_fold_ROC(0,0,0)
    #print roc_0_0_0
    auc_0_0_0 = multiclass_storage.get_fold_evaluation_result(0,0,0,0)
    #print auc_0_0_0
    return roc_0_0_0, auc_0_0_0


if __name__=='__main__':
	print('Evaluation CrossValidationMulticlassStorage')
	evaluation_cross_validation_multiclass_storage(*parameter_list[0])



def prepare_data(use_toy=True):
    from os.path import exists
    from tools.load import LoadMatrix
    lm=LoadMatrix()

    if not use_toy and exists('../data/../mldata/uci-20070111-optdigits.mat'):
        from scipy.io import loadmat

        mat = loadmat('../data/../mldata/uci-20070111-optdigits.mat')['int0'].astype(float)
        X = mat[:-1,:]
        Y = mat[-1,:]
        isplit = X.shape[1]/2
        traindat = X[:,:isplit]
        label_traindat = Y[:isplit]
        testdat = X[:, isplit:]
        label_testdat = Y[isplit:]
    else:
        traindat = lm.load_numbers('../data/fm_train_real.dat')
        testdat  = lm.load_numbers('../data/fm_test_real.dat')
        label_traindat = lm.load_labels('../data/label_train_multiclass.dat')
        label_testdat = None

    return [traindat, label_traindat, testdat, label_testdat]



#!/usr/bin/env python
import numpy
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')
label_traindat = lm.load_labels('../data/label_train_twoclass.dat')

parameter_list = [[traindat,testdat,label_traindat,0.9,1e-3],[traindat,testdat,label_traindat,0.8,1e-2]]

def features_director_dot_modular (fm_train_real, fm_test_real,
		label_train_twoclass, C, epsilon):
	try:
		from modshogun import DirectorDotFeatures
		from modshogun import RealVector
	except ImportError:
		print("recompile shogun with --enable-swig-directors")
		return

	class NumpyFeatures(DirectorDotFeatures):

		# variables
		data=numpy.empty((1,1))

		# constructor
		def __init__(self, d):
			DirectorDotFeatures.__init__(self)
			self.data = d

		# overloaded methods
		def add_to_dense_sgvec(self, alpha, vec_idx1, vec2, abs):
			if abs:
				vec2+=alpha*numpy.abs(self.data[:,vec_idx1])
			else:
				vec2+=alpha*self.data[:,vec_idx1]

		def dot(self, vec_idx1, df, vec_idx2):
			return numpy.dot(self.data[:,vec_idx1], df.get_computed_dot_feature_vector(vec_idx2))

		def dense_dot_sgvec(self, vec_idx1, vec2):
			return numpy.dot(self.data[:,vec_idx1], vec2[0:vec2.vlen])

		def get_num_vectors(self):
			return self.data.shape[1]

		def get_dim_feature_space(self):
			return self.data.shape[0]

		# operators
	#	def __add__(self, other):
	#		return NumpyFeatures(self.data+other.data)

	#	def __sub__(self, other):
	#		return NumpyFeatures(self.data-other.data)

	#	def __iadd__(self, other):
	#		return NumpyFeatures(self.data+other.data)

	#	def __isub__(self, other):
	#		return NumpyFeatures(self.data-other.data)


	#from modshogun import RealFeatures, SparseRealFeatures, BinaryLabels
	#from modshogun import LibLinear, L2R_L2LOSS_SVC_DUAL
	#from modshogun import Math_init_random
	#Math_init_random(17)

	#feats_train=RealFeatures(fm_train_real)
	#feats_test=RealFeatures(fm_test_real)
	#labels=BinaryLabels(label_train_twoclass)

	#dfeats_train=NumpyFeatures(fm_train_real)
	#dfeats_test=NumpyFeatures(fm_test_real)
	#dlabels=BinaryLabels(label_train_twoclass)

	#print feats_train.get_computed_dot_feature_matrix()
	#print dfeats_train.get_computed_dot_feature_matrix()

	#svm=LibLinear(C, feats_train, labels)
	#svm.set_liblinear_solver_type(L2R_L2LOSS_SVC_DUAL)
	#svm.set_epsilon(epsilon)
	#svm.set_bias_enabled(True)
	#svm.train()

	#svm.set_features(feats_test)
	#svm.apply().get_labels()
	#predictions = svm.apply()

	#dfeats_train.__disown__()
	#dfeats_train.parallel.set_num_threads(1)
	#dsvm=LibLinear(C, dfeats_train, dlabels)
	#dsvm.set_liblinear_solver_type(L2R_L2LOSS_SVC_DUAL)
	#dsvm.set_epsilon(epsilon)
	#dsvm.set_bias_enabled(True)
	#dsvm.train()

	#dfeats_test.__disown__()
	#dfeats_test.parallel.set_num_threads(1)
	#dsvm.set_features(dfeats_test)
	#dsvm.apply().get_labels()
	#dpredictions = dsvm.apply()

	#return predictions, svm, predictions.get_labels()

if __name__=='__main__':
	print('DirectorLinear')
	features_director_dot_modular(*parameter_list[0])



#!/usr/bin/env python
traindna = '../data/fm_train_dna.dat'
testdna = '../data/fm_test_dna.dat'

parameter_list = [[traindna,testdna,3,0,False],[traindna,testdna,4,0,False]]

def distance_manhattenword_modular (train_fname=traindna,test_fname=testdna,order=3,gap=0,reverse=False):
	from modshogun import StringCharFeatures, StringWordFeatures, DNA
	from modshogun import SortWordString, ManhattanWordDistance, CSVFile

	charfeat=StringCharFeatures(CSVFile(train_fname), DNA)
	feats_train=StringWordFeatures(charfeat.get_alphabet())
	feats_train.obtain_from_char(charfeat, order-1, order, gap, reverse)
	preproc=SortWordString()
	preproc.init(feats_train)
	feats_train.add_preprocessor(preproc)
	feats_train.apply_preprocessor()

	charfeat=StringCharFeatures(CSVFile(test_fname), DNA)
	feats_test=StringWordFeatures(charfeat.get_alphabet())
	feats_test.obtain_from_char(charfeat, order-1, order, gap, reverse)
	feats_test.add_preprocessor(preproc)
	feats_test.apply_preprocessor()

	distance=ManhattanWordDistance(feats_train, feats_train)

	dm_train=distance.get_distance_matrix()
	distance.init(feats_train, feats_test)
	dm_test=distance.get_distance_matrix()
	return dm_train,dm_test

if __name__=='__main__':
	print('ManhattanWordDistance')
	distance_manhattenword_modular(*parameter_list[0])



from numpy import *
import matplotlib.pyplot as p
import os, sys, inspect
path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../tools'))

if not path in sys.path:
    sys.path.insert(1, path)
del path
from generate_circle_data import circle_data
cir=circle_data()
number_of_points_for_circle1=42
number_of_points_for_circle2=122
row_vector=2
data=cir.generate_data(number_of_points_for_circle1,number_of_points_for_circle2,row_vector)
d=zeros((row_vector,number_of_points_for_circle1))
d2=zeros((row_vector,number_of_points_for_circle2))
d=[data[i][0:number_of_points_for_circle1] for i in range(0,row_vector)]
d2=[data[i][number_of_points_for_circle1:(number_of_points_for_circle1+number_of_points_for_circle2)] for i in range(0,row_vector)]
p.plot(d[1][:],d[0][:],'x',d2[1][:],d2[0][:],'o')
p.title('input data')
p.show()



parameter_list = [[data,0.01,1.0], [data,0.05,2.0]]
def preprocessor_kernelpca_modular (data, threshold, width):

	from modshogun import RealFeatures
	from modshogun import KernelPCA
	from modshogun import GaussianKernel
	features = RealFeatures(data)
	kernel=GaussianKernel(features,features,width)
	preprocessor=KernelPCA(kernel)
	preprocessor.init(features)
	preprocessor.set_target_dim(2)
	#X=preprocessor.get_transformation_matrix()
	X2=preprocessor.apply_to_feature_matrix(features)
	lx0=len(X2)
	modified_d1=zeros((lx0,number_of_points_for_circle1))
	modified_d2=zeros((lx0,number_of_points_for_circle2))
	modified_d1=[X2[i][0:number_of_points_for_circle1] for i in range(lx0)]
	modified_d2=[X2[i][number_of_points_for_circle1:(number_of_points_for_circle1+number_of_points_for_circle2)] for i in range(lx0)]
	p.plot(modified_d1[0][:],modified_d1[1][:],'o',modified_d2[0][:],modified_d2[1][:],'x')
	p.title('final data')
	p.show()
	return features

if __name__=='__main__':
	print('KernelPCA')
	preprocessor_kernelpca_modular(*parameter_list[0])



from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindna=lm.load_dna('../data/fm_train_dna.dat')
testdna=lm.load_dna('../data/fm_test_dna.dat')
trainlabel=lm.load_labels('../data/label_train_dna.dat')
parameter_list=[[traindna,testdna,trainlabel,10,5,5,7],
		[traindna,testdna,trainlabel,11,6,6,8]]

def kernel_localityimprovedstring (fm_train_dna=traindna,fm_test_dna=testdna,
				 label_train_dna=trainlabel,size_cache=10,
				 length=5,inner_degree=5,outer_degree=7):

	sg('set_features', 'TRAIN', fm_train_dna, 'DNA')
	sg('set_features', 'TEST', fm_test_dna, 'DNA')
	sg('set_kernel', 'LIK', 'CHAR', size_cache, length, inner_degree, outer_degree)
	km=sg('get_kernel_matrix', 'TRAIN')
	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('LocalityImprovedString')
	kernel_localityimprovedstring(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()

traindat = lm.load_numbers('../data/fm_train_real.dat')
testdat = lm.load_numbers('../data/fm_test_real.dat')

parameter_list = [[traindat,testdat,1.1],[traindat,testdat,1.2]]

def kernel_sparse_linear_modular (fm_train_real=traindat,fm_test_real=testdat,scale=1.1):
	from modshogun import SparseRealFeatures
	from modshogun import LinearKernel, AvgDiagKernelNormalizer

	feats_train=SparseRealFeatures(fm_train_real)
	feats_test=SparseRealFeatures(fm_test_real)

	kernel=LinearKernel()
	kernel.set_normalizer(AvgDiagKernelNormalizer(scale))
	kernel.init(feats_train, feats_train)
	km_train=kernel.get_kernel_matrix()

	kernel.init(feats_train, feats_test)
	km_test=kernel.get_kernel_matrix()
	return km_train,km_test,kernel

if __name__=='__main__':
	print('SparseLinear')
	kernel_sparse_linear_modular(*parameter_list[0])



"""
Test Distance
"""

from modshogun import *

import util


def _evaluate (indata):
	prefix='distance_'
	feats=util.get_features(indata, prefix)

	dfun=eval(indata[prefix+'name'])
	dargs=util.get_args(indata, prefix)
	distance=dfun(feats['train'], feats['train'], *dargs)

	dm_train=max(abs(
		indata[prefix+'matrix_train']-distance.get_distance_matrix()).flat)
	distance.init(feats['train'], feats['test'])
	dm_test=max(abs(
		indata[prefix+'matrix_test']-distance.get_distance_matrix()).flat)

	return util.check_accuracy(
		indata[prefix+'accuracy'], dm_train=dm_train, dm_test=dm_test)


########################################################################
# public
########################################################################

def test (indata):
	return _evaluate(indata)




#!/usr/bin/env python

parameter_list=[[100, 250, 10, 2]]

def structure_plif_hmsvm_bmrm (num_examples, example_length, num_features, num_noise_features):
	from modshogun import RealMatrixFeatures, TwoStateModel, DualLibQPBMSOSVM, StructuredAccuracy

	model = TwoStateModel.simulate_data(num_examples, example_length, num_features, num_noise_features)
	sosvm = DualLibQPBMSOSVM(model, model.get_labels(), 5000.0)

	sosvm.train()
	#print sosvm.get_w()

	predicted = sosvm.apply(model.get_features())
	evaluator = StructuredAccuracy()
	acc = evaluator.evaluate(predicted, model.get_labels())
	#print('Accuracy = %.4f' % acc)

if __name__ == '__main__':
	print("PLiF HMSVM BMRM")
	structure_plif_hmsvm_bmrm(*parameter_list[0])



"""
Shogun demo

Fernando J. Iglesias Garcia
"""

import numpy as np
import matplotlib as mpl
import pylab
import util

from scipy import linalg
from modshogun import QDA
from modshogun import RealFeatures, MulticlassLabels

# colormap
cmap = mpl.colors.LinearSegmentedColormap('color_classes',
	{'red':   [(0, 1,  1),
		   (1, .7, .7)],
	 'green': [(0, 1, 1),
	           (1, .7, .7)],
	 'blue':  [(0, 1, 1),
	           (1, .7, .7)]})
pylab.cm.register_cmap(cmap = cmap)

# Generate data from Gaussian distributions
def gen_data():
	np.random.seed(0)
	covs = np.array([[[0., -1. ], [2.5,  .7]],
			 [[3., -1.5], [1.2, .3]],
			 [[ 2,  0  ], [ .0,  1.5 ]]])
	X = np.r_[np.dot(np.random.randn(N, dim), covs[0]) + np.array([-4, 3]),
		  np.dot(np.random.randn(N, dim), covs[1]) + np.array([-1, -5]),
		  np.dot(np.random.randn(N, dim), covs[2]) + np.array([3, 4])];
	Y = np.hstack((np.zeros(N), np.ones(N), 2*np.ones(N)))
	return X, Y

def plot_data(qda, X, y, y_pred, ax):
	X0, X1, X2 = X[y == 0], X[y == 1], X[y == 2]

	# Correctly classified
	tp = (y == y_pred)
	tp0, tp1, tp2 = tp[y == 0], tp[y == 1], tp[y == 2]
	X0_tp, X1_tp, X2_tp = X0[tp0], X1[tp1], X2[tp2]

	# Misclassified
	X0_fp, X1_fp, X2_fp = X0[tp0 != True], X1[tp1 != True], X2[tp2 != True]

	# Class 0 data
	pylab.plot(X0_tp[:, 0], X0_tp[:, 1], 'o', color = cols[0])
	pylab.plot(X0_fp[:, 0], X0_fp[:, 1], 's', color = cols[0])
	m0 = qda.get_mean(0)
	pylab.plot(m0[0], m0[1], 'o', color = 'black', markersize = 8)

	# Class 1 data
	pylab.plot(X1_tp[:, 0], X1_tp[:, 1], 'o', color = cols[1])
	pylab.plot(X1_fp[:, 0], X1_fp[:, 1], 's', color = cols[1])
	m1 = qda.get_mean(1)
	pylab.plot(m1[0], m1[1], 'o', color = 'black', markersize = 8)

	# Class 2 data
	pylab.plot(X2_tp[:, 0], X2_tp[:, 1], 'o', color = cols[2])
	pylab.plot(X2_fp[:, 0], X2_fp[:, 1], 's', color = cols[2])
	m2 = qda.get_mean(2)
	pylab.plot(m2[0], m2[1], 'o', color = 'black', markersize = 8)

def plot_cov(plot, mean, cov, color):
	v, w = linalg.eigh(cov)
	u = w[0] / linalg.norm(w[0])
	angle = np.arctan(u[1] / u[0])	# rad
	angle = 180 * angle / np.pi	# degrees
	# Filled gaussian at 2 standard deviation
	ell = mpl.patches.Ellipse(mean, 2*v[0]**0.5, 2*v[1]**0.5, 180 + angle, color = color)
	ell.set_clip_box(plot.bbox)
	ell.set_alpha(0.5)
	plot.add_artist(ell)

def plot_regions(qda):
	nx, ny = 500, 500
	x_min, x_max = pylab.xlim()
	y_min, y_max = pylab.ylim()
	xx, yy = np.meshgrid(np.linspace(x_min, x_max, nx),
			     np.linspace(y_min, y_max, ny))
	dense = RealFeatures(np.array((np.ravel(xx), np.ravel(yy))))
	dense_labels = qda.apply(dense).get_labels()
	Z = dense_labels.reshape(xx.shape)
	pylab.pcolormesh(xx, yy, Z)
	pylab.contour(xx, yy, Z, linewidths = 3, colors = 'k')

# Number of classes
M = 3
# Number of samples of each class
N = 300
# Dimension of the data
dim = 2

cols = ['blue', 'green', 'red']

fig = pylab.figure()
ax  = fig.add_subplot(111)
pylab.title('Quadratic Discrimant Analysis')

X, y = gen_data()

labels = MulticlassLabels(y)
features = RealFeatures(X.T)
qda = QDA(features, labels, 1e-4, True)
qda.train()
ypred = qda.apply().get_labels()

plot_data(qda, X, y, ypred, ax)
for i in range(M):
	plot_cov(ax, qda.get_mean(i), qda.get_cov(i), cols[i])
plot_regions(qda)

pylab.connect('key_press_event', util.quit)
pylab.show()



"""Generator for Preprocessors"""

import shogun.Library as library
from shogun.Kernel import GaussianKernel, CommWordStringKernel, \
	CommUlongStringKernel

import fileop
import featop
import dataop
import category


def _compute (feats, params):
	"""Perform computations on kernel using preprocessors.

	@param name name of the kernel
	@param feats features of the kernel
	@return dict of testcase data ready to be written to file
	"""

	output=fileop.get_output(category.KERNEL, params)

	fun=eval(params['name']+'Kernel')
	if params.has_key('args'):
		kernel=fun(feats['train'], feats['train'], *params['args']['val'])
	else:
		kernel=fun(feats['train'], feats['train'])

	output['kernel_matrix_train']=kernel.get_kernel_matrix()
	kernel.init(feats['train'], feats['test'])
	output['kernel_matrix_test']=kernel.get_kernel_matrix()

	return output


def _run_string_complex (ftype):
	"""Run preprocessor applied on complex StringFeatures.

	@param ftype Feature type, like Word
	"""

	params={
		'name': 'Comm'+ftype+'String',
		'accuracy': 1e-9,
		'feature_class': 'string_complex',
		'feature_type': ftype,
		'data': dataop.get_dna()
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	# string_complex gets preproc added implicitely on Word/Ulong feats
	output=_compute(feats, params)

	params={
		'name': 'Sort'+ftype+'String'
	}
	output.update(fileop.get_output(category.PREPROC, params))

	fileop.write(category.PREPROC, output)


def _run_real (name, args=None):
	"""Run preprocessor applied on RealFeatures.

	@param name name of the preprocessor
	@param args argument list (in a dict) for the preprocessor
	"""

	params={
		'name': 'Gaussian',
		'accuracy': 1e-8,
		'data': dataop.get_rand(),
		'feature_class': 'simple',
		'feature_type': 'Real',
		'args': {'key': ('width',), 'val': (1.2,)}
	}
	feats=featop.get_features(
		params['feature_class'], params['feature_type'], params['data'])
	if args:
		feats=featop.add_preproc(name, feats, *args['val'])
	else:
		feats=featop.add_preproc(name, feats)

	output=_compute(feats, params)

	params={ 'name': name }
	if args:
		params['args']=args

	output.update(fileop.get_output(category.PREPROC, params))

	fileop.write(category.PREPROC, output)


def run():
	"""Run generator for all preprocessors."""

	_run_real('LogPlusOne')
	_run_real('NormOne')
	_run_real('PruneVarSubMean', {'key': ('divide',), 'val': (False,)})
	_run_real('PruneVarSubMean', {'key': ('divide',), 'val': (True,)})

	_run_string_complex('Word')
	_run_string_complex('Ulong')

#	_run_norm_derivative_lem3()
#	_run_pcacut()



#!/usr/bin/env python

parameter_list=[[100, 250, 10, 2]]

def structure_plif_hmsvm_mosek (num_examples, example_length, num_features, num_noise_features):
	from modshogun import RealMatrixFeatures, TwoStateModel, StructuredAccuracy

	try:
		from modshogun import PrimalMosekSOSVM
	except ImportError:
		print("Mosek not available")
		return

	model = TwoStateModel.simulate_data(num_examples, example_length, num_features, num_noise_features)
	sosvm = PrimalMosekSOSVM(model, model.get_labels())

	sosvm.train()
	#print(sosvm.get_w())

	predicted = sosvm.apply(model.get_features())
	evaluator = StructuredAccuracy()
	acc = evaluator.evaluate(predicted, model.get_labels())
	#print('Accuracy = %.4f' % acc)

if __name__ == '__main__':
	print("PLiF HMSVM Mosek")
	structure_plif_hmsvm_mosek(*parameter_list[0])



#!/usr/bin/env python
from tools.load import LoadMatrix
lm=LoadMatrix()
data=lm.load_numbers('../data/fm_train_real.dat')
label=lm.load_numbers('../data/label_train_twoclass.dat')

parameter_list=[[data,label]]

def features_io_modular (fm_train_real, label_train_twoclass):
	import numpy
	from modshogun import SparseRealFeatures, RealFeatures, MulticlassLabels
	from modshogun import GaussianKernel
	from modshogun import LibSVMFile, CSVFile, BinaryFile, HDF5File

	feats=SparseRealFeatures(fm_train_real)
	feats2=SparseRealFeatures()

	f=BinaryFile("fm_train_sparsereal.bin","w")
	feats.save(f)

	f=LibSVMFile("fm_train_sparsereal.ascii","w")
	feats.save(f)

	f=BinaryFile("fm_train_sparsereal.bin")
	feats2.load(f)

	f=LibSVMFile("fm_train_sparsereal.ascii")
	feats2.load(f)

	feats=RealFeatures(fm_train_real)
	feats2=RealFeatures()

	f=BinaryFile("fm_train_real.bin","w")
	feats.save(f)

	f=HDF5File("fm_train_real.h5","w", "/data/doubles")
	feats.save(f)

	f=CSVFile("fm_train_real.ascii","w")
	feats.save(f)

	f=BinaryFile("fm_train_real.bin")
	feats2.load(f)
	#print("diff binary", numpy.max(numpy.abs(feats2.get_feature_matrix().flatten()-fm_train_real.flatten())))

	f=CSVFile("fm_train_real.ascii")
	feats2.load(f)
	#print("diff ascii", numpy.max(numpy.abs(feats2.get_feature_matrix().flatten()-fm_train_real.flatten())))

	lab=MulticlassLabels(numpy.array([0.0,1.0,2.0,3.0]))
	lab2=MulticlassLabels()
	f=CSVFile("label_train_twoclass.ascii","w")
	lab.save(f)

	f=BinaryFile("label_train_twoclass.bin","w")
	lab.save(f)

	f=HDF5File("label_train_real.h5","w", "/data/labels")
	lab.save(f)

	f=CSVFile("label_train_twoclass.ascii")
	lab2.load(f)

	f=BinaryFile("label_train_twoclass.bin")
	lab2.load(f)

	f=HDF5File("fm_train_real.h5","r", "/data/doubles")
	feats2.load(f)
	#print(feats2.get_feature_matrix())
	f=HDF5File("label_train_real.h5","r", "/data/labels")
	lab2.load(f)
	#print(lab2.get_labels())

	#clean up
	import os
	for f in ['fm_train_sparsereal.bin','fm_train_sparsereal.ascii',
			'fm_train_real.bin','fm_train_real.h5','fm_train_real.ascii',
			'label_train_real.h5', 'label_train_twoclass.ascii','label_train_twoclass.bin']:
		os.unlink(f)
	return feats, feats2, lab, lab2

if __name__=='__main__':
	print('Features IO')
	features_io_modular(*parameter_list[0])



#!/usr/bin/env python
strings=['hey','guys','string']

parameter_list=[[strings,0,2,0,False]]

def features_string_word_modular (strings, start, order, gap, rev):
	from modshogun import StringCharFeatures, StringWordFeatures, RAWBYTE
	from numpy import array, uint16

	#create string features
	cf=StringCharFeatures(strings, RAWBYTE)
	wf=StringWordFeatures(RAWBYTE)

	wf.obtain_from_char(cf, start, order, gap, rev)

	#and output several stats
	#print("max string length", wf.get_max_vector_length())
	#print("number of strings", wf.get_num_vectors())
	#print("length of first string", wf.get_vector_length(0))
	#print("string[2]", wf.get_feature_vector(2))
	#print("strings", wf.get_features())

	#replace string 0
	wf.set_feature_vector(array([1,2,3,4,5], dtype=uint16), 0)

	#print("strings", wf.get_features())
	return wf.get_features(), wf

if __name__=='__main__':
	print('StringWordFeatures')
	features_string_word_modular(*parameter_list[0])



#!/usr/bin/env python

traindat = '../data/fm_train_real.dat'
testdat = '../data/fm_test_real.dat'
label_traindat = '../data/label_train_multiclass.dat'

parameter_list = [[traindat,testdat,label_traindat,3]]

def metric_lmnn_modular(train_fname=traindat,test_fname=testdat,label_train_fname=label_traindat,k=3):
	try:
		from modshogun import RealFeatures,MulticlassLabels,LMNN,KNN,CSVFile
	except ImportError:
		return

	# wrap features and labels into Shogun objects
	feats_train=RealFeatures(CSVFile(train_fname))
	feats_test=RealFeatures(CSVFile(test_fname))
	labels=MulticlassLabels(CSVFile(label_train_fname))

	# LMNN
	lmnn=LMNN(feats_train,labels,k)
	lmnn.train()
	lmnn_distance=lmnn.get_distance()

	# perform classification with KNN
	knn=KNN(k,lmnn_distance,labels)
	knn.train()
	output=knn.apply(feats_test).get_labels()

	return lmnn,output

if __name__=='__main__':
	print('LMNN')
	metric_lmnn_modular(*parameter_list[0])



"""
Test Regression
"""

from sg import sg
from numpy import double
import util


def _set_regression (indata):
	prefix='regression_'
	sg('threads', indata[prefix+'num_threads'])
	sg('set_labels', 'TRAIN', double(indata[prefix+'labels']))

	rname=util.fix_regression_name_inconsistency(indata[prefix+'name'])
	print rname
	sg('new_regression', rname)


def _train (indata):
	if indata['regression_type']=='svm':
		sg('c', double(indata['regression_C']))
		sg('svm_epsilon', indata['regression_epsilon'])
		sg('svr_tube_epsilon', indata['regression_tube_epsilon'])
	elif indata['regression_type']=='kernelmachine':
		sg('krr_tau', indata['regression_tau'])
	else:
		raise StandardError, 'Incomplete regression data.'

	sg('train_regression')


def _evaluate (indata):
	alphas=0
	bias=0
	sv=0

	if indata.has_key('regression_bias'):
		[bias, weights]=sg('get_svm')
		weights=weights.T
		bias=abs(bias-indata['regression_bias'])
		for item in weights[0].tolist():
			alphas+=item
		alphas=abs(alphas-indata['regression_alpha_sum'])
		for item in weights[1].tolist():
			sv+=item
		sv=abs(sv-indata['regression_sv_sum'])

	classified=max(abs(sg('classify')-indata['regression_classified']))

	return util.check_accuracy(indata['regression_accuracy'],
		alphas=alphas, bias=bias, support_vectors=sv, classified=classified)


########################################################################
# public
########################################################################

def test (indata):
	try:
		util.set_features(indata, 'kernel_')
	except NotImplementedError, e:
		print e
		return True

	util.set_and_train_kernel(indata)

	try:
		_set_regression(indata)
	except RuntimeError, e:
		print "%s is disabled/unavailable!" % indata['name']
		return True

	try:
		_train(indata)
	except StandardError, e:
		print e
		return False

	return _evaluate(indata)




#!/usr/bin/env python
parameter_list = [['features_string_file_char_modular.py']]

def features_string_file_char_modular (fname):
	from modshogun import StringFileCharFeatures, RAWBYTE
	f = StringFileCharFeatures(fname, RAWBYTE)
	#print("strings", f.get_features())
	return f

if __name__=='__main__':
    print('Compressing StringCharFileFeatures')
    features_string_file_char_modular(*parameter_list[0])



import modshogun as sg
import data

# load data
feature_matrix = data.swissroll()
# create features instance
features = sg.RealFeatures(feature_matrix)

# create Linear Local Tangent Space Alignment converter instance
converter = sg.LinearLocalTangentSpaceAlignment()

# set target dimensionality
converter.set_target_dim(2)
# set number of neighbors
converter.set_k(10)
# set number of threads
converter.parallel.set_num_threads(2)
# set nullspace shift (optional)
converter.set_nullspace_shift(-1e-6)

# compute embedding with Linear Local Tangent Space Alignment method
embedding = converter.embed(features)



from tools.load import LoadMatrix
from sg import sg
from numpy import *
num=100
labelstrain=concatenate((-ones([1,num]), ones([1,num])),1)[0]
featuretrain=concatenate((random.normal(size=(2,num))-1,random.normal(size=(2,num))+1),1)
parameter_list=[[1.,labelstrain,featuretrain],
				[1.,labelstrain,featuretrain]]

def mkl_twoclass (weight=1.,
		labels=labelstrain,features=featuretrain):

	sg('c', 10.)
	sg('new_classifier', 'MKL_CLASSIFICATION')

	sg('set_labels', 'TRAIN', labels)
	sg('add_features', 'TRAIN', features)
	sg('add_features', 'TRAIN', features)
	sg('add_features', 'TRAIN', features)

	sg('set_kernel', 'COMBINED', 100)
	sg('add_kernel', weight, 'GAUSSIAN', 'REAL', 100, 100.)
	sg('add_kernel', weight, 'GAUSSIAN', 'REAL', 100, 10.)
	sg('add_kernel', weight, 'GAUSSIAN', 'REAL', 100, 1.)
	sg('train_classifier')
	[bias, alphas]=sg('get_svm');

	km=sg('get_kernel_matrix', 'TEST')
	return km

if __name__=='__main__':
	print('MKL_TWOCLASS')
	mkl_twoclass(*parameter_list[0])




#!/usr/bin/env python
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# Written (W) 2011-2012 Heiko Strathmann
# Copyright (C) 2011 Berlin Institute of Technology and Max-Planck-Society
#

parameter_list=[[None]]

def modelselection_parameter_tree_modular (dummy):
    from modshogun import ParameterCombination
    from modshogun import ModelSelectionParameters, R_EXP, R_LINEAR
    from modshogun import PowerKernel
    from modshogun import GaussianKernel
    from modshogun import DistantSegmentsKernel
    from modshogun import MinkowskiMetric

    root=ModelSelectionParameters()

    combinations=root.get_combinations()
    combinations.get_num_elements()

    c=ModelSelectionParameters('C');
    root.append_child(c)
    c.build_values(1, 11, R_EXP)

    power_kernel=PowerKernel()

    # print all parameter available for modelselection
    # Dont worry if yours is not included but, write to the mailing list
    #power_kernel.print_modsel_params()

    param_power_kernel=ModelSelectionParameters('kernel', power_kernel)
    root.append_child(param_power_kernel)

    param_power_kernel_degree=ModelSelectionParameters('degree')
    param_power_kernel_degree.build_values(1, 1, R_EXP)
    param_power_kernel.append_child(param_power_kernel_degree)

    metric1=MinkowskiMetric(10)

    # print all parameter available for modelselection
    # Dont worry if yours is not included but, write to the mailing list
    #metric1.print_modsel_params()

    param_power_kernel_metric1=ModelSelectionParameters('distance', metric1)

    param_power_kernel.append_child(param_power_kernel_metric1)

    param_power_kernel_metric1_k=ModelSelectionParameters('k')
    param_power_kernel_metric1_k.build_values(1, 12, R_LINEAR)
    param_power_kernel_metric1.append_child(param_power_kernel_metric1_k)

    gaussian_kernel=GaussianKernel()

    # print all parameter available for modelselection
    # Dont worry if yours is not included but, write to the mailing list
    #gaussian_kernel.print_modsel_params()

    param_gaussian_kernel=ModelSelectionParameters('kernel', gaussian_kernel)

    root.append_child(param_gaussian_kernel)

    param_gaussian_kernel_width=ModelSelectionParameters('width')
    param_gaussian_kernel_width.build_values(1, 2, R_EXP)
    param_gaussian_kernel.append_child(param_gaussian_kernel_width)

    ds_kernel=DistantSegmentsKernel()

    # print all parameter available for modelselection
    # Dont worry if yours is not included but, write to the mailing list
    #ds_kernel.print_modsel_params()

    param_ds_kernel=ModelSelectionParameters('kernel', ds_kernel)

    root.append_child(param_ds_kernel)

    param_ds_kernel_delta=ModelSelectionParameters('delta')
    param_ds_kernel_delta.build_values(1, 2, R_EXP)
    param_ds_kernel.append_child(param_ds_kernel_delta)

    param_ds_kernel_theta=ModelSelectionParameters('theta')
    param_ds_kernel_theta.build_values(1, 2, R_EXP)
    param_ds_kernel.append_child(param_ds_kernel_theta)

    #	root.print_tree()
    combinations=root.get_combinations()
    #	for i in range(combinations.get_num_elements()):
    #		combinations.get_element(i).print_tree()

    return


if __name__=='__main__':
    print('ModelSelection ParameterTree')
    modelselection_parameter_tree_modular(*parameter_list[0])






from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
parameter_list=[[traindat,10,3],[traindat,11,4]]
def clustering_hierarchical (fm_train=traindat, size_cache=10,merges=3):

	sg('set_features', 'TRAIN', fm_train)
	sg('set_distance', 'EUCLIDEAN', 'REAL')
	sg('new_clustering', 'HIERARCHICAL')
	sg('train_clustering', merges)

	[merge_distance, pairs]=sg('get_clustering')
	return [merge_distance, pairs]

if __name__=='__main__':
	print('Hierarchical')
	clustering_hierarchical(*parameter_list[0])



"""
Test Clustering
"""

from sg import sg
import util


def _set_clustering (indata):
	cname=util.fix_clustering_name_inconsistency(indata['clustering_name'])
	sg('new_clustering', cname)


def _train (indata, prefix):
	if indata.has_key(prefix+'max_iter'):
		max_iter=indata[prefix+'max_iter']
	else:
		max_iter=1000

	if indata.has_key(prefix+'k'):
		first_arg=indata[prefix+'k']
	elif indata.has_key(prefix+'merges'):
		first_arg=indata[prefix+'merges']
	else:
		raise StandardError, 'Incomplete clustering data.'

	sg('train_clustering', first_arg, max_iter)


def _evaluate (indata, prefix):
	if indata.has_key(prefix+'radi'):
		[radi, centers]=sg('get_clustering')
		radi=max(abs(radi.T[0]-indata[prefix+'radi']))
		centers=max(abs(centers-indata[prefix+'centers']).flat)

		return util.check_accuracy(indata[prefix+'accuracy'],
			radi=radi, centers=centers)

	elif indata.has_key(prefix+'merge_distance'):
		[merge_distances, pairs]=sg('get_clustering')
		merge_distances=max(abs(merge_distances.T[0]- \
			indata[prefix+'merge_distance']))
		pairs=max(abs(pairs-indata[prefix+'pairs']).flat)

		return util.check_accuracy(indata[prefix+'accuracy'],
			merge_distances=merge_distances, pairs=pairs)

	else:
		raise StandardError, 'Incomplete clustering data.'

########################################################################
# public
########################################################################

def test (indata):
	try:
		util.set_features(indata, 'distance_')
	except NotImplementedError, e:
		print e
		return True

	util.set_and_train_distance(indata)
	_set_clustering(indata)

	prefix='clustering_'
	try:
		_train(indata, prefix)
		return _evaluate(indata, prefix)
	except StandardError, e:
		print e
		return False




"""A package to generate testcases for the Shogun toolbox

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation in version 3.

Written (W) 2007-2008 Sebastian Henschel
Copyright (C) 2007-2008 Fraunhofer Institute FIRST and Max-Planck-Society
"""

import sys
import os
import numpy.random as random

from shogun.Library import Math_init_random
from dataop import INIT_RANDOM
from fileop import clean_dir_outdata

CATEGORIES=['classifier', 'clustering', 'distance', 'distribution', 'kernel', \
	'regression', 'preproc']

def run (argv):
	"""
	Run all individual generators or only one if present in
	argument list.
	"""

	# put some constantness into randomness
	Math_init_random(INIT_RANDOM)
	random.seed(INIT_RANDOM)

	arglen=len(argv)
	if arglen==2: # run given category
		if argv[1]=='clear':
			clean_dir_outdata()
		else:
			try:
				__import__(argv[1], globals(), locals());
			except ImportError:
				raise ImportError, 'Unknown category ' + argv[1]

			category=eval(argv[1])
			category.run()
	else:
		# run given category by calling self again, one by one
		# this is due to an issue somewhere with classifiers (atm) and
		# 'static randomness'

		if arglen==1:
			command=argv[0]
			cats=CATEGORIES
		else:
			command=argv.pop(0)
			cats=argv

		for cat in cats:
			if not cat in CATEGORIES:
				cats=', '.join(CATEGORIES)
				msg="Unknown category: %s\nTry one of these: %s\n"%(cat, cats)
				sys.stderr.write(msg)
				sys.exit(1)

			ret=os.system('%s %s' % (command, cat))
			if ret!=0:
				sys.exit(ret)




from tools.load import LoadMatrix
from sg import sg
lm=LoadMatrix()

traindat=lm.load_numbers('../data/fm_train_real.dat')
testdat=lm.load_numbers('../data/fm_test_real.dat')
trainlabel=lm.load_labels('../data/label_train_regression.dat')
parameter_list=[[traindat,testdat,trainlabel,10,2.1,1.2,1e-5,1e-2],
		[traindat,testdat,trainlabel,11,2.3,1.3,1e-6,1e-3]]

def regression_libsvr (fm_train=traindat,fm_test=testdat,
		label_train=trainlabel,size_cache=10,width=2.1,
		C=1.2,epsilon=1e-5,tube_epsilon=1e-2):

	sg('set_features', 'TRAIN', fm_train)
	sg('set_kernel', 'GAUSSIAN', 'REAL', size_cache, width)

	sg('set_labels', 'TRAIN', label_train)
	sg('new_regression', 'LIBSVR')
	sg('svr_tube_epsilon', tube_epsilon)
	sg('c', C)
	sg('train_regression')

	sg('set_features', 'TEST', fm_test)
	result=sg('classify')
	return result

if __name__=='__main__':
	print('LibSVR')
	regression_libsvr(*parameter_list[0])



#!/usr/bin/env python
data = '../data/fm_train_real.dat'
parameter_list = [[data,10],[data,20]]

def converter_diffusionmaps_modular (data_fname,t):
	try:
		from modshogun import RealFeatures, DiffusionMaps, GaussianKernel, CSVFile

		features = RealFeatures(CSVFile(data_fname))

		converter = DiffusionMaps()
		converter.set_target_dim(1)
		converter.set_kernel(GaussianKernel(10,10.0))
		converter.set_t(t)
		converter.apply(features)

		return features
	except ImportError:
		print('No Eigen3 available')

if __name__=='__main__':
	print('DiffusionMaps')
	converter_diffusionmaps_modular(*parameter_list[0])

